@@ -28,6 +28,7 @@
 #include "katehighlight.h"
 #include "kateconfig.h"
 #include "katefactory.h"
+#include "kateautoindent.h"
 
 #include <kdebug.h>
 #include <kglobal.h>
@@ -39,6 +40,7 @@
 #include <qtimer.h>
 #include <qtextcodec.h>
 #include <qcstring.h>
+#include <qdatetime.h>
 
 /**
  * loader block size, load 256 kb at once per default
@@ -92,6 +94,8 @@ class KateFileLoader
       , lastWasEndOfLine (true) // at start of file, we had a virtual newline
       , lastWasR (false) // we have not found a \r as last char
       , m_eol (-1) // no eol type detected atm
+      , m_twoByteEncoding (QString(codec->name()) == "ISO-10646-UCS-2")
+      , m_binary (false)
     {
     }
 
@@ -110,9 +114,12 @@ class KateFileLoader
         int c = m_file.readBlock (m_buffer.data(), m_buffer.size());
 
         if (c > 0)
+        {
+          processNull (c);
           m_text = m_decoder->toUnicode (m_buffer, c);
+        }
 
-        m_eof = m_file.atEnd();
+        m_eof = (c == -1) || (c == 0) || (m_text.length() == 0) || m_file.atEnd();
 
         for (uint i=0; i < m_text.length(); i++)
         {
@@ -148,6 +155,9 @@ class KateFileLoader
     // eol mode ? autodetected on open(), -1 for no eol found in the first block!
     inline int eol () const { return m_eol; }
 
+    // binary ?
+    inline bool binary () const { return m_binary; }
+
     // internal unicode data array
     inline const QChar *unicode () const { return m_text.unicode(); }
 
@@ -166,14 +176,22 @@ class KateFileLoader
           {
             int c = m_file.readBlock (m_buffer.data(), m_buffer.size());
 
+            uint readString = 0;
             if (c > 0)
+            {
+              processNull (c);
+
+              QString str (m_decoder->toUnicode (m_buffer, c));
+              readString = str.length();
+
               m_text = m_text.mid (m_lastLineStart, m_position-m_lastLineStart)
-                       + m_decoder->toUnicode (m_buffer, c);
+                       + str;
+            }
             else
               m_text = m_text.mid (m_lastLineStart, m_position-m_lastLineStart);
 
             // is file completly read ?
-            m_eof = m_file.atEnd();
+            m_eof = (c == -1) || (c == 0) || (readString == 0) || m_file.atEnd();
 
             // recalc current pos and last pos
             m_position -= m_lastLineStart;
@@ -240,6 +258,34 @@ class KateFileLoader
       }
     }
 
+    // this nice methode will kill all 0 bytes (or double bytes)
+    // and remember if this was a binary or not ;)
+    void processNull (uint length)
+    {
+      if (m_twoByteEncoding)
+      {
+        for (uint i=1; i < length; i+=2)
+        {
+          if ((m_buffer[i] == 0) && (m_buffer[i-1] == 0))
+          {
+            m_binary = true;
+            m_buffer[i] = ' ';
+          }
+        }
+      }
+      else
+      {
+        for (uint i=0; i < length; i++)
+        {
+          if (m_buffer[i] == 0)
+          {
+            m_binary = true;
+            m_buffer[i] = ' ';
+          }
+        }
+      }
+    }
+
   private:
     QFile m_file;
     QByteArray m_buffer;
@@ -251,6 +297,8 @@ class KateFileLoader
     bool lastWasEndOfLine;
     bool lastWasR;
     int m_eol;
+    bool m_twoByteEncoding;
+    bool m_binary;
 };
 
 /**
@@ -269,6 +317,7 @@ KateBuffer::KateBuffer(KateDocument *doc)
    m_cacheReadError(false),
    m_cacheWriteError(false),
    m_loadingBorked (false),
+   m_binary (false),
    m_highlight (0),
    m_regionTree (this),
    m_tabWidth (8),
@@ -276,8 +325,6 @@ KateBuffer::KateBuffer(KateDocument *doc)
    m_lineHighlighted (0),
    m_maxDynamicContexts (KATE_MAX_DYNAMIC_CONTEXTS)
 {
-  connect( &m_regionTree,SIGNAL(setLineVisible(unsigned int, bool)), this,SLOT(setLineVisible(unsigned int,bool)));
-
   clear();
 }
 
@@ -289,6 +336,10 @@ KateBuffer::~KateBuffer()
   // DELETE ALL BLOCKS, will free mem
   for (uint i=0; i < m_blocks.size(); i++)
     delete m_blocks[i];
+
+  // release HL
+  if (m_highlight)
+    m_highlight->release();
 }
 
 void KateBuffer::editStart ()
@@ -315,7 +366,9 @@ void KateBuffer::editEnd ()
     return;
 
   // hl update !!!
-  if ((editTagLineStart <= editTagLineEnd) && (editTagLineEnd <= m_lineHighlighted))
+  if ( m_highlight && !m_highlight->noHighlighting()
+       && (editTagLineStart <= editTagLineEnd)
+       && (editTagLineEnd <= m_lineHighlighted))
   {
     // look one line too far, needed for linecontinue stuff
     editTagLineEnd++;
@@ -405,6 +458,7 @@ void KateBuffer::clear()
   m_cacheWriteError = false;
   m_cacheReadError = false;
   m_loadingBorked = false;
+  m_binary = false;
 
   m_lineHighlightedMax = 0;
   m_lineHighlighted = 0;
@@ -484,6 +538,11 @@ bool KateBuffer::openFile (const QString &m_file)
     m_lineHighlightedMax = m_lines;
   }
 
+  // binary?
+  m_binary = file.binary ();
+
+  kdDebug (13020) << "LOADING DONE" << endl;
+
   return !m_loadingBorked;
 }
 
@@ -788,10 +847,36 @@ void KateBuffer::setTabWidth (uint w)
   }
 }
 
-void KateBuffer::setHighlight(KateHighlighting *highlight)
+void KateBuffer::setHighlight(uint hlMode)
 {
-  m_highlight = highlight;
-  invalidateHighlighting();
+  KateHighlighting *h = KateHlManager::self()->getHl(hlMode);
+
+   // aha, hl will change
+  if (h != m_highlight)
+  {
+    bool invalidate = !h->noHighlighting();
+
+    if (m_highlight)
+    {
+      m_highlight->release();
+      invalidate = true;
+    }
+
+    h->use();
+
+    // try to set indentation
+    if (!h->indentation().isEmpty())
+      m_doc->config()->setIndentationMode (KateAutoIndent::modeNumber(h->indentation()));
+
+    m_highlight = h;
+
+    if (invalidate)
+      invalidateHighlighting();
+
+    // inform the document that the hl was really changed
+    // needed to update attributes and more ;)
+    m_doc->bufferHlChanged ();
+  }
 }
 
 void KateBuffer::invalidateHighlighting()
@@ -800,7 +885,7 @@ void KateBuffer::invalidateHighlighting()
   m_lineHighlighted = 0;
 }
 
-bool KateBuffer::doHighlight(KateBufBlock *buf, uint startLine, uint endLine, bool invalidate)
+bool KateBuffer::doHighlight (KateBufBlock *buf, uint startLine, uint endLine, bool invalidate)
 {
   // no hl around, no stuff to do
   if (!m_highlight)
@@ -810,7 +895,9 @@ bool KateBuffer::doHighlight(KateBufBlock *buf, uint startLine, uint endLine, bo
   if (startLine >= (buf->startLine()+buf->lines()))
     return false;
 
-  kdDebug (13020) << "NEED HL, LINESTART: " << startLine << " LINEEND: " << endLine << endl;
+  QTime t;
+  t.start();
+  kdDebug (13020) << "HIGHLIGHTED START --- NEED HL, LINESTART: " << startLine << " LINEEND: " << endLine << endl;
   kdDebug (13020) << "HL UNTIL LINE: " << m_lineHighlighted << " MAX: " << m_lineHighlightedMax << endl;
   kdDebug (13020) << "HL DYN COUNT: " << KateHlManager::self()->countDynamicCtxs() << " MAX: " << m_maxDynamicContexts << endl;
 
@@ -883,7 +970,7 @@ bool KateBuffer::doHighlight(KateBufBlock *buf, uint startLine, uint endLine, bo
     // current line
     KateTextLine::Ptr textLine = buf->line(current_line);
 
-    QMemArray<signed char> foldingList;
+    QMemArray<uint> foldingList;
     bool ctxChanged = false;
 
     m_highlight->doHighlight (prevLine, textLine, &foldingList, &ctxChanged);
@@ -972,8 +1059,9 @@ bool KateBuffer::doHighlight(KateBufBlock *buf, uint startLine, uint endLine, bo
       // add folding start to the list !
       if (newIn)
       {
-        foldingList.resize (foldingList.size() + 1, QGArray::SpeedOptim);
-        foldingList[foldingList.size()-1] = 1;
+        foldingList.resize (foldingList.size() + 2, QGArray::SpeedOptim);
+        foldingList[foldingList.size()-2] = 1;
+        foldingList[foldingList.size()-1] = 0;
       }
 
       // calculate how much end folding symbols must be added to the list !
@@ -990,20 +1078,48 @@ bool KateBuffer::doHighlight(KateBufBlock *buf, uint startLine, uint endLine, bo
 
       if (remIn > 0)
       {
-        foldingList.resize (foldingList.size() + remIn, QGArray::SpeedOptim);
+        foldingList.resize (foldingList.size() + (remIn*2), QGArray::SpeedOptim);
 
-        for (uint z= foldingList.size()-remIn; z < foldingList.size(); z++)
+        for (uint z= foldingList.size()-(remIn*2); z < foldingList.size(); z=z+2)
+        {
           foldingList[z] = -1;
+          foldingList[z+1] = 0;
+        }
+      }
+    }
+    bool foldingColChanged=false;
+    bool foldingChanged = false; //!(foldingList == textLine->foldingListArray());
+    if (foldingList.size()!=textLine->foldingListArray().size()) {
+      foldingChanged=true;
+    } else {
+      QMemArray<uint>::ConstIterator it=foldingList.begin();
+      QMemArray<uint>::ConstIterator it1=textLine->foldingListArray();
+      bool markerType=true;
+      for(;it!=foldingList.end();++it,++it1) {
+        if  (markerType) {
+          if ( ((*it)!=(*it1))) {
+            foldingChanged=true;
+            foldingColChanged=false;
+            break;
+          }
+        } else {
+            if ((*it)!=(*it1)) {
+              foldingColChanged=true;
+            }
+        }
+        markerType=!markerType;
       }
     }
 
-    bool foldingChanged = !(foldingList == textLine->foldingListArray());
-
-    if (foldingChanged)
+    if (foldingChanged || foldingColChanged) {
       textLine->setFoldingList(foldingList);
-
+      if (foldingChanged==false){
+        textLine->setFoldingColumnsOutdated(textLine->foldingColumnsOutdated() | foldingColChanged);
+      } else textLine->setFoldingColumnsOutdated(false);
+    }
     bool retVal_folding = false;
-    m_regionTree.updateLine (current_line + buf->startLine(), &foldingList, &retVal_folding, foldingChanged);
+    //perhaps make en enums out of the change flags
+    m_regionTree.updateLine (current_line + buf->startLine(), &foldingList, &retVal_folding, foldingChanged,foldingColChanged);
 
     codeFoldingUpdate = codeFoldingUpdate | retVal_folding;
 
@@ -1027,29 +1143,28 @@ bool KateBuffer::doHighlight(KateBufBlock *buf, uint startLine, uint endLine, bo
   if (codeFoldingUpdate)
     emit codeFoldingUpdated();
 
+  kdDebug (13020) << "HIGHLIGHTED END --- NEED HL, LINESTART: " << startLine << " LINEEND: " << endLine << endl;
+  kdDebug (13020) << "HL UNTIL LINE: " << m_lineHighlighted << " MAX: " << m_lineHighlightedMax << endl;
+  kdDebug (13020) << "HL DYN COUNT: " << KateHlManager::self()->countDynamicCtxs() << " MAX: " << m_maxDynamicContexts << endl;
+  kdDebug (13020) << "TIME TAKEN: " << t.elapsed() << endl;
+
   // if we are at the last line of the block + we still need to continue
   // return the need of that !
   return stillcontinue && ((current_line+1) == buf->lines());
 }
 
-void KateBuffer::setLineVisible(unsigned int lineNr, bool visible)
-{
-   KateBufBlock *buf = findBlock(lineNr);
-
-   if (!buf)
-     return;
-
-   KateTextLine::Ptr l = buf->line(lineNr - buf->startLine());
-
-   if (l && (l->isVisible () != visible))
-   {
-     l->setVisible(visible);
-
-     buf->markDirty ();
-   }
+void KateBuffer::codeFoldingColumnUpdate(unsigned int lineNr) {
+  KateTextLine::Ptr line=plainLine(lineNr);
+  if (!line) return;
+  if (line->foldingColumnsOutdated()) {
+    line->setFoldingColumnsOutdated(false);
+    bool tmp;
+    QMemArray<uint> folding=line->foldingListArray();
+    m_regionTree.updateLine(lineNr,&folding,&tmp,true,false);
+  }
 }
 
-// BEGIN KateBufBlock
+//BEGIN KateBufBlock
 
 KateBufBlock::KateBufBlock ( KateBuffer *parent, KateBufBlock *prev, KateBufBlock *next,
                              KateFileLoader *stream )
@@ -1110,7 +1225,7 @@ KateBufBlock::~KateBufBlock ()
 
   // if we have some swapped data allocated, free it now or never
   if (m_vmblock)
-    m_parent->vm()->free(m_vmblock);
+    KateFactory::self()->vm()->free(m_vmblock);
 
   // remove me from the list I belong
   KateBufBlockList::remove (this);
@@ -1175,15 +1290,15 @@ void KateBufBlock::fillBlock (KateFileLoader *stream)
 
   if (swap)
   {
-    m_vmblock = m_parent->vm()->allocate(size);
+    m_vmblock = KateFactory::self()->vm()->allocate(size);
     m_vmblockSize = size;
 
     if (!rawData.isEmpty())
     {
-      if (!m_parent->vm()->copyBlock(m_vmblock, rawData.data(), 0, size))
+      if (!KateFactory::self()->vm()->copyBlock(m_vmblock, rawData.data(), 0, size))
       {
         if (m_vmblock)
-          m_parent->vm()->free(m_vmblock);
+          KateFactory::self()->vm()->free(m_vmblock);
 
         m_vmblock = 0;
         m_vmblockSize = 0;
@@ -1254,7 +1369,7 @@ void KateBufBlock::markDirty ()
     {
       // if we have some swapped data allocated which is dirty, free it now
       if (m_vmblock)
-        m_parent->vm()->free(m_vmblock);
+        KateFactory::self()->vm()->free(m_vmblock);
 
       m_vmblock = 0;
       m_vmblockSize = 0;
@@ -1273,7 +1388,7 @@ void KateBufBlock::swapIn ()
   QByteArray rawData (m_vmblockSize);
 
   // what to do if that fails ?
-  if (!m_parent->vm()->copyBlock(rawData.data(), m_vmblock, 0, rawData.size()))
+  if (!KateFactory::self()->vm()->copyBlock(rawData.data(), m_vmblock, 0, rawData.size()))
     m_parent->m_cacheReadError = true;
 
   // reserve mem, keep realloc away on push_back
@@ -1317,15 +1432,15 @@ void KateBufBlock::swapOut ()
     for (uint i=0; i < m_lines; i++)
       buf = m_stringList[i]->dump (buf, haveHl);
 
-    m_vmblock = m_parent->vm()->allocate(rawData.size());
+    m_vmblock = KateFactory::self()->vm()->allocate(rawData.size());
     m_vmblockSize = rawData.size();
 
     if (!rawData.isEmpty())
     {
-      if (!m_parent->vm()->copyBlock(m_vmblock, rawData.data(), 0, rawData.size()))
+      if (!KateFactory::self()->vm()->copyBlock(m_vmblock, rawData.data(), 0, rawData.size()))
       {
         if (m_vmblock)
-          m_parent->vm()->free(m_vmblock);
+          KateFactory::self()->vm()->free(m_vmblock);
 
         m_vmblock = 0;
         m_vmblockSize = 0;
@@ -1344,9 +1459,9 @@ void KateBufBlock::swapOut ()
   KateBufBlockList::remove (this);
 }
 
-// END KateBufBlock
+//END KateBufBlock
 
-// BEGIN KateBufBlockList
+//BEGIN KateBufBlockList
 
 KateBufBlockList::KateBufBlockList ()
  : m_count (0),
@@ -1424,6 +1539,6 @@ void KateBufBlockList::removeInternal (KateBufBlock *buf)
   buf->list = 0;
 }
 
-// END KateBufBlockList
+//END KateBufBlockList
 
 // kate: space-indent on; indent-width 2; replace-tabs on;
