@@ -3,7 +3,7 @@
  *  This file is part of the KDE libraries
  *  Copyright (c) 2001 Waldo Bastian <bastian@kde.org>
  *
- * $Id: kconf_update.cpp,v 1.7 2001/08/02 21:25:42 waba Exp $
+ * $Id: kconf_update.cpp,v 1.34 2005/01/21 12:21:54 waba Exp $
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -33,22 +33,31 @@
 #include <klocale.h>
 #include <kcmdlineargs.h>
 #include <kglobal.h>
-#include <kstddirs.h>
+#include <kstandarddirs.h>
 #include <kaboutdata.h>
 #include <kinstance.h>
 #include <ktempfile.h>
 
 static KCmdLineOptions options[] =
 {
+        { "debug", I18N_NOOP("Keep output results from scripts"), 0 },
+	{ "check <update-file>", I18N_NOOP("Check whether config file itself requires updating"), 0 },
 	{ "+[file]", I18N_NOOP("File to read update instructions from"), 0 },
-        { 0, 0, 0 }
+        KCmdLineLastOption
 };
 
 class KonfUpdate
 {
 public:
    KonfUpdate();
-   QStringList findDirtyUpdateFiles();
+   ~KonfUpdate();
+   QStringList findUpdateFiles(bool dirtyOnly);
+
+   QTextStream &log();
+
+   bool checkFile(const QString &filename);
+   void checkGotFile(const QString &_file, const QString &id);
+
    bool updateFile(const QString &filename);
 
    void gotId(const QString &_id);
@@ -58,8 +67,10 @@ public:
    void gotKey(const QString &_key);
    void gotRemoveKey(const QString &_key);
    void gotAllKeys();
+   void gotAllGroups();
    void gotOptions(const QString &_options);
    void gotScript(const QString &_script);
+   void gotScriptArguments(const QString &_arguments);
    void resetOptions();
 
    void copyGroup(KConfigBase *cfg1, const QString &grp1, 
@@ -69,13 +80,15 @@ protected:
    KConfig *config;
    QString currentFilename;
    bool skip;
+   bool debug;
    QString id;
 
    QString oldFile;
    QString newFile;
+   QString newFileName;
    KConfig *oldConfig1; // Config to read keys from.
-   KSimpleConfig *oldConfig2; // Config to delete keys from.
-   KSimpleConfig *newConfig;
+   KConfig *oldConfig2; // Config to delete keys from.
+   KConfig *newConfig;
 
    QString oldGroup;
    QString newGroup;
@@ -84,10 +97,18 @@ protected:
 
    bool m_bCopy;
    bool m_bOverwrite;
+   bool m_bUseConfigInfo;
+   QString m_arguments;
+   QTextStream *m_textStream;
+   QFile *m_file;
+   QString m_line;
+   int m_lineCount;
 };
 
 KonfUpdate::KonfUpdate()
+ : m_textStream(0), m_file(0)
 {
+   bool updateAll = false;
    oldConfig1 = 0;
    oldConfig2 = 0;
    newConfig = 0;
@@ -96,7 +117,23 @@ KonfUpdate::KonfUpdate()
 
    QStringList updateFiles;
    KCmdLineArgs *args=KCmdLineArgs::parsedArgs();
-   if (args->count())
+   
+   debug = args->isSet("debug");
+
+   m_bUseConfigInfo = false;
+   if (args->isSet("check"))
+   {
+      m_bUseConfigInfo = true;
+      QString file = locate("data", "kconf_update/"+QFile::decodeName(args->getOption("check")));
+      if (file.isEmpty())
+      {
+         qWarning("File '%s' not found.", args->getOption("check").data());
+         log() << "File '" << QFile::decodeName(args->getOption("check")) << "' passed on command line not found" << endl;
+         return;
+      }
+      updateFiles.append(file);
+   }
+   else if (args->count())
    {
       for(int i = 0; i < args->count(); i++)
       {
@@ -108,20 +145,68 @@ KonfUpdate::KonfUpdate()
    }
    else
    {
-      updateFiles = findDirtyUpdateFiles();
+      if (config->readBoolEntry("autoUpdateDisabled", false))
+         return;
+      updateFiles = findUpdateFiles(true);
+      updateAll = true;
    }
+
    for(QStringList::ConstIterator it = updateFiles.begin();
        it != updateFiles.end();
        ++it)
    {
       QString file = *it;
-qWarning("File = %s", file.latin1());
       updateFile(file);
    }
+
+   config->setGroup(QString::null);
+   if (updateAll && !config->readBoolEntry("updateInfoAdded", false))
+   {
+       config->writeEntry("updateInfoAdded", true);
+       updateFiles = findUpdateFiles(false);
+
+       for(QStringList::ConstIterator it = updateFiles.begin();
+           it != updateFiles.end();
+           ++it)
+       {
+           QString file = *it;
+           checkFile(file);
+       }
+       updateFiles.clear();
+   }
 }
 
+KonfUpdate::~KonfUpdate()
+{
+   delete config;
+   delete m_file;
+   delete m_textStream;
+}
 
-QStringList KonfUpdate::findDirtyUpdateFiles()
+QTextStream &
+KonfUpdate::log()
+{
+   if (!m_textStream)
+   {
+      QString file = locateLocal("data", "kconf_update/update.log");
+      m_file = new QFile(file);
+      if (m_file->open(IO_WriteOnly | IO_Append))
+      {
+        m_textStream = new QTextStream(m_file);
+      }
+      else
+      {
+        // Error
+        m_textStream = new QTextStream(stderr, IO_WriteOnly);
+      }
+   }
+   
+   (*m_textStream) << QDateTime::currentDateTime().toString( Qt::ISODate ) << " ";
+   
+   return *m_textStream;
+}
+
+QStringList KonfUpdate::findUpdateFiles(bool dirtyOnly)
 {
    QStringList result;
    QStringList list = KGlobal::dirs()->findAllResources("data", "kconf_update/*.upd", false, true);
@@ -139,7 +224,8 @@ QStringList KonfUpdate::findDirtyUpdateFiles()
          config->setGroup(file);
          time_t ctime = config->readUnsignedLongNumEntry("ctime");
          time_t mtime = config->readUnsignedLongNumEntry("mtime");
-         if ((ctime != buff.st_ctime) || (mtime != buff.st_mtime))
+         if (!dirtyOnly ||
+             (ctime != buff.st_ctime) || (mtime != buff.st_mtime))
          {
             result.append(*it);
          }
@@ -148,11 +234,67 @@ QStringList KonfUpdate::findDirtyUpdateFiles()
    return result;
 }
 
+bool KonfUpdate::checkFile(const QString &filename)
+{
+   currentFilename = filename;
+   int i = currentFilename.findRev('/');
+   if (i != -1) 
+      currentFilename = currentFilename.mid(i+1);
+   skip = true;
+   QFile file(filename);
+   if (!file.open(IO_ReadOnly))
+      return false;
+
+   QTextStream ts(&file);
+   ts.setEncoding(QTextStream::Latin1);
+   int lineCount = 0;
+   resetOptions();
+   QString id;
+   while(!ts.atEnd())
+   {
+      QString line = ts.readLine().stripWhiteSpace();
+      lineCount++;
+      if (line.isEmpty() || (line[0] == '#'))
+         continue;
+      if (line.startsWith("Id="))
+         id = currentFilename+":"+line.mid(3);
+      else if (line.startsWith("File="))
+         checkGotFile(line.mid(5), id);
+   }
+  
+   return true;
+}
+
+void KonfUpdate::checkGotFile(const QString &_file, const QString &id)
+{
+   QString file;
+   int i = _file.find(',');
+   if (i == -1)
+   {
+      file = _file.stripWhiteSpace();
+   }
+   else
+   {
+      file = _file.mid(i+1).stripWhiteSpace();
+   }
+
+//   qDebug("File %s, id %s", file.latin1(), id.latin1());
+
+   KSimpleConfig cfg(file);
+   cfg.setGroup("$Version");
+   QStringList ids = cfg.readListEntry("update_info");
+   if (ids.contains(id))
+       return;
+   ids.append(id);
+   cfg.writeEntry("update_info", ids);
+}
+
 /**
  * Syntax:
  * # Comment
  * Id=id
  * File=oldfile[,newfile]
+ * AllGroups
  * Group=oldgroup[,newgroup]
  * RemoveGroup=oldgroup
  * Options=[copy,][overwrite,]
@@ -160,6 +302,7 @@ QStringList KonfUpdate::findDirtyUpdateFiles()
  * RemoveKey=ldkey
  * AllKeys
  * Keys= [Options](AllKeys|(Key|RemoveKey)*)
+ * ScriptArguments=arguments
  * Script=scriptfile[,interpreter]
  *
  * Sequence:
@@ -176,53 +319,64 @@ bool KonfUpdate::updateFile(const QString &filename)
    if (!file.open(IO_ReadOnly))
       return false;
 
+   log() << "Checking update-file '" << filename << "' for new updates" << endl; 
+
    QTextStream ts(&file);
    ts.setEncoding(QTextStream::Latin1);
-   int lineCount = 0;
+   m_lineCount = 0;
    resetOptions();
    while(!ts.atEnd())
    {
-      QString line = ts.readLine().stripWhiteSpace();
-      lineCount++;
-      if (line.isEmpty() || (line[0] == '#'))
+      m_line = ts.readLine().stripWhiteSpace();
+      m_lineCount++;
+      if (m_line.isEmpty() || (m_line[0] == '#'))
          continue;
-      if (line.startsWith("Id="))
-         gotId(line.mid(3));
+      if (m_line.startsWith("Id="))
+         gotId(m_line.mid(3));
       else if (skip)
          continue;
-      else if (line.startsWith("Options="))
-         gotOptions(line.mid(8));
-      else if (line.startsWith("File="))
-         gotFile(line.mid(5));
-      else if (line.startsWith("Group="))
-         gotGroup(line.mid(6));
-      else if (line.startsWith("RemoveGroup="))
+      else if (m_line.startsWith("Options="))
+         gotOptions(m_line.mid(8));
+      else if (m_line.startsWith("File="))
+         gotFile(m_line.mid(5));
+      else if (m_line.startsWith("Group="))
+         gotGroup(m_line.mid(6));
+      else if (m_line.startsWith("RemoveGroup="))
       {
-         gotRemoveGroup(line.mid(12));
+         gotRemoveGroup(m_line.mid(12));
          resetOptions();
       }
-      else if (line.startsWith("Script="))
+      else if (m_line.startsWith("Script="))
       {
-         gotScript(line.mid(7));
+         gotScript(m_line.mid(7));
          resetOptions();
       }
-      else if (line.startsWith("Key="))
+      else if (m_line.startsWith("ScriptArguments="))
+         gotScriptArguments(m_line.mid(16));
+      else if (m_line.startsWith("Key="))
       {
-         gotKey(line.mid(4));
+         gotKey(m_line.mid(4));
          resetOptions();
       }
-      else if (line.startsWith("RemoveKey="))
+      else if (m_line.startsWith("RemoveKey="))
       {
-         gotRemoveKey(line.mid(10));
+         gotRemoveKey(m_line.mid(10));
          resetOptions();
       }
-      else if (line == "AllKeys")
+      else if (m_line == "AllKeys")
       {
          gotAllKeys();
          resetOptions();
       }
+      else if (m_line == "AllGroups")
+      {
+         gotAllGroups();
+         resetOptions();
+      }
       else
-         qWarning("%s:%d parse error '%s'", filename.latin1(), lineCount, line.latin1());
+      {
+         log() << currentFilename << ": parse error in line " << m_lineCount << " : '" << m_line << "'" << endl;
+      }
    }
    // Flush.
    gotId(QString::null);
@@ -236,32 +390,44 @@ bool KonfUpdate::updateFile(const QString &filename)
    return true;
 }
 
+
+
 void KonfUpdate::gotId(const QString &_id)
 {
+   if (!id.isEmpty() && !skip)
+   {
+       config->setGroup(currentFilename);
+       QStringList ids = config->readListEntry("done");
+       if (!ids.contains(id))
+       {
+          ids.append(id);
+          config->writeEntry("done", ids);
+          config->sync();
+       }
+   }
+
    // Flush pending changes
    gotFile(QString::null);
 
    config->setGroup(currentFilename);
    QStringList ids = config->readListEntry("done");
-   if (!id.isEmpty())
-   {
-      if (!ids.contains(_id))
-         ids.append(id);
-      else
-         qWarning("Id '%s' was already in done-list!", id.latin1());
-      config->writeEntry("done", ids);
-      config->sync();
-   }
    if (!_id.isEmpty())
    {
-      if (ids.contains(_id))
-      {
-         skip = true;
-qWarning("Skipping '%s'", _id.latin1());
-         return;
-      }
-      skip = false;
-      id = _id;
+       if (ids.contains(_id))
+       {
+          //qDebug("Id '%s' was already in done-list", _id.latin1());
+          if (!m_bUseConfigInfo)
+          {
+             skip = true;
+             return;
+          }
+       }
+       skip = false;
+       id = _id;
+       if (m_bUseConfigInfo)
+          log() << currentFilename << ": Checking update '" << _id << "'" << endl;
+       else
+          log() << currentFilename << ": Found new update '" << _id << "'" << endl;
    }
 }
 
@@ -275,15 +441,43 @@ void KonfUpdate::gotFile(const QString &_file)
       // Close old file.
       delete oldConfig1;
       oldConfig1 = 0;
+
+      oldConfig2->setGroup("$Version");
+      QStringList ids = oldConfig2->readListEntry("update_info");
+      QString cfg_id = currentFilename + ":" + id;
+      if (!ids.contains(cfg_id) && !skip)
+      {
+         ids.append(cfg_id);
+         oldConfig2->writeEntry("update_info", ids);
+      }
       oldConfig2->sync();
       delete oldConfig2;
       oldConfig2 = 0;
+      
+      QString file = locateLocal("config", oldFile);
+      struct stat s_buf;
+      if (stat(QFile::encodeName(file), &s_buf) == 0)
+      {
+         if (s_buf.st_size == 0)
+         {
+            // Delete empty file.
+            unlink(QFile::encodeName(file));
+         }   
+      }
 
       oldFile = QString::null;
    }
    if (!newFile.isEmpty())
    {
       // Close new file.
+      newConfig->setGroup("$Version");
+      QStringList ids = newConfig->readListEntry("update_info");
+      QString cfg_id = currentFilename + ":" + id;
+      if (!ids.contains(cfg_id) && !skip)
+      {
+         ids.append(cfg_id);
+         newConfig->writeEntry("update_info", ids);
+      }
       newConfig->sync();
       delete newConfig;
       newConfig = 0;
@@ -307,21 +501,42 @@ void KonfUpdate::gotFile(const QString &_file)
    
    if (!oldFile.isEmpty())
    {
-      oldConfig1 = new KConfig(oldFile, true, false);
-      oldConfig2 = new KSimpleConfig(oldFile);
+      oldConfig2 = new KConfig(oldFile, false, false);
+      QString cfg_id = currentFilename + ":" + id;
+      oldConfig2->setGroup("$Version");
+      QStringList ids = oldConfig2->readListEntry("update_info");
+      if (ids.contains(cfg_id))
+      {
+         skip = true;
+         newFile = QString::null;
+         log() << currentFilename << ": Skipping update '" << id << "'" << endl;
+      }
+
       if (!newFile.isEmpty())
       {
-         newConfig = new KSimpleConfig(newFile);
+         newConfig = new KConfig(newFile, false, false);
+         newConfig->setGroup("$Version");
+         ids = newConfig->readListEntry("update_info");
+         if (ids.contains(cfg_id))
+         {
+            skip = true;
+            log() << currentFilename << ": Skipping update '" << id << "'" << endl;
+         }
       }
       else
       {
          newConfig = oldConfig2;
       }
+
+      oldConfig1 = new KConfig(oldFile, true, false);
    }
    else
    {
       newFile = QString::null;
    }
+   newFileName = newFile;
+   if (newFileName.isEmpty())
+      newFileName = oldFile;
 }
 
 void KonfUpdate::gotGroup(const QString &_group)
@@ -345,7 +560,7 @@ void KonfUpdate::gotRemoveGroup(const QString &_group)
 
    if (!oldConfig1)
    {
-      qWarning("RemoveGroup without file specification.");
+      log() << currentFilename << ": !! RemoveGroup without previous File specification in line " << m_lineCount << " : '" << m_line << "'" << endl;
       return;
    }
 
@@ -353,7 +568,7 @@ void KonfUpdate::gotRemoveGroup(const QString &_group)
       return;
    // Delete group.
    oldConfig2->deleteGroup(oldGroup, true);
-   qWarning("Removing group %s (FORCED)", oldGroup.isEmpty() ? "<empty>" : oldGroup.latin1());
+   log() << currentFilename << ": RemoveGroup removes group " << oldFile << ":" << oldGroup << endl;
 }
 
 
@@ -373,12 +588,12 @@ void KonfUpdate::gotKey(const QString &_key)
 
    if (oldKey.isEmpty() || newKey.isEmpty())
    {
-      qWarning("Invalid key.");
+      log() << currentFilename << ": !! Key specifies invalid key in line " << m_lineCount << " : '" << m_line << "'" << endl;
       return;
    }
    if (!oldConfig1)
    {
-      qWarning("Key without file specification.");
+      log() << currentFilename << ": !! Key without previous File specification in line " << m_lineCount << " : '" << m_line << "'" << endl;
       return;
    }
    oldConfig1->setGroup(oldGroup);
@@ -388,10 +603,10 @@ void KonfUpdate::gotKey(const QString &_key)
    newConfig->setGroup(newGroup);
    if (!m_bOverwrite && newConfig->hasKey(newKey))
    {
-      qWarning("Skipping %s", newKey.latin1());
+      log() << currentFilename << ": Skipping " << newFileName << ":" << newGroup << ":" << newKey << ", already exists."<< endl;
       return;
    }
-qWarning("Write %s -> %s", newKey.latin1(), value.isEmpty() ? "<empty>" : value.latin1());
+   log() << currentFilename << ": Updating " << newFileName << ":" << newGroup << ":" << newKey << " to '" << value << "'" << endl;
    newConfig->writeEntry(newKey, value);
 
    if (m_bCopy)
@@ -404,8 +619,10 @@ qWarning("Write %s -> %s", newKey.latin1(), value.isEmpty() ? "<empty>" : value.
       return; // Don't delete!
    oldConfig2->setGroup(oldGroup);
    oldConfig2->deleteEntry(oldKey, false);
-   if (oldConfig2->deleteGroup(oldGroup, false)) // Delete group if empty.
-      qWarning("Removing group %s", oldGroup.isEmpty() ? "<empty>" : oldGroup.latin1());
+   log() << currentFilename << ": Removing " << oldFile << ":" << oldGroup << ":" << oldKey << ", moved." << endl;
+   if (oldConfig2->deleteGroup(oldGroup, false)) { // Delete group if empty.
+      log() << currentFilename << ": Removing empty group " << oldFile << ":" << oldGroup << endl;
+   }
 }
 
 void KonfUpdate::gotRemoveKey(const QString &_key)
@@ -414,33 +631,34 @@ void KonfUpdate::gotRemoveKey(const QString &_key)
 
    if (oldKey.isEmpty())
    {
-      qWarning("Invalid key.");
+      log() << currentFilename << ": !! RemoveKey specifies invalid key in line " << m_lineCount << " : '" << m_line << "'" << endl;
       return;
    }
 
    if (!oldConfig1)
    {
-      qWarning("RemoveKey without file specification.");
+      log() << currentFilename << ": !! Key without previous File specification in line " << m_lineCount << " : '" << m_line << "'" << endl;
       return;
    }
 
    oldConfig1->setGroup(oldGroup);
    if (!oldConfig1->hasKey(oldKey))
       return;
-qWarning("Remove Key '%s'/'%s'", oldGroup.isEmpty() ? "empty" : oldGroup.latin1(), oldKey.latin1());
+   log() << currentFilename << ": RemoveKey removes " << oldFile << ":" << oldGroup << ":" << oldKey << endl;
 
    // Delete old entry
    oldConfig2->setGroup(oldGroup);
    oldConfig2->deleteEntry(oldKey, false);
-   if (oldConfig2->deleteGroup(oldGroup, false)) // Delete group if empty.
-      qWarning("Removing group %s", oldGroup.isEmpty() ? "<empty>" : oldGroup.latin1());
+   if (oldConfig2->deleteGroup(oldGroup, false)) { // Delete group if empty.
+      log() << currentFilename << ": Removing empty group " << oldFile << ":" << oldGroup << endl;
+   }
 }
 
 void KonfUpdate::gotAllKeys()
 {
    if (!oldConfig1)
    {
-      qWarning("AllKeys without file specification.");
+      log() << currentFilename << ": !! AllKeys without previous File specification in line " << m_lineCount << " : '" << m_line << "'" << endl;
       return;
    }
 
@@ -452,6 +670,24 @@ void KonfUpdate::gotAllKeys()
    }
 }
 
+void KonfUpdate::gotAllGroups()
+{
+   if (!oldConfig1)
+   {
+      log() << currentFilename << ": !! AllGroups without previous File specification in line " << m_lineCount << " : '" << m_line << "'" << endl;
+      return;
+   }
+
+   QStringList allGroups = oldConfig1->groupList();
+   for(QStringList::ConstIterator it = allGroups.begin();
+       it != allGroups.end(); ++it)
+   {
+     oldGroup = *it;
+     newGroup = oldGroup;
+     gotAllKeys();
+   }
+}
+
 void KonfUpdate::gotOptions(const QString &_options)
 {
    QStringList options = QStringList::split(',', _options);
@@ -470,15 +706,21 @@ void KonfUpdate::gotOptions(const QString &_options)
 void KonfUpdate::copyGroup(KConfigBase *cfg1, const QString &grp1, 
                            KConfigBase *cfg2, const QString &grp2)
 {
+   cfg1->setGroup(grp1);
    cfg2->setGroup(grp2);
    QMap<QString, QString> list = cfg1->entryMap(grp1);
    for(QMap<QString, QString>::Iterator it = list.begin();
        it != list.end(); ++it)
    {
-      cfg2->writeEntry(it.key(), it.data());
+      cfg2->writeEntry(it.key(), cfg1->readEntry(it.key()));
    }
 }
 
+void KonfUpdate::gotScriptArguments(const QString &_arguments)
+{
+   m_arguments = _arguments;
+}
+
 void KonfUpdate::gotScript(const QString &_script)
 {
    QString script, interpreter;
@@ -493,49 +735,34 @@ void KonfUpdate::gotScript(const QString &_script)
       interpreter = _script.mid(i+1).stripWhiteSpace();
    }
 
-   if (!oldConfig1)
-   {
-      qWarning("Script without file specification.");
-      return;
-   }
 
    if (script.isEmpty())
    {
-      qWarning("No script specified.");
+      log() << currentFilename << ": !! Script fails to specifiy filename in line " << m_lineCount << " : '" << m_line << "'" << endl;
+      skip = true;
       return;
    } 
-   qWarning("Running script '%s'", script.latin1());
+
+
 
    QString path = locate("data","kconf_update/"+script);
    if (path.isEmpty())
    {
-      qWarning("Script '%s' not found.", script.latin1());
-      return;
+      if (interpreter.isEmpty())
+         path = locate("lib", "kconf_update_bin/"+script);
+
+      if (path.isEmpty())
+      {
+        log() << currentFilename << ": !! Script '" << script << "' not found in line " << m_lineCount << " : '" << m_line << "'" << endl;
+        skip = true;
+        return;
+      }
    }
 
-   KTempFile tmp1;
-   tmp1.setAutoDelete(true);
-   KTempFile tmp2;
-   tmp2.setAutoDelete(true);
-   KSimpleConfig cfg(tmp1.name());
-
-   if (oldGroup.isEmpty())
-   {
-       // Write all entries to tmpFile;
-       QStringList grpList = oldConfig1->groupList();
-       for(QStringList::ConstIterator it = grpList.begin();
-           it != grpList.end();
-           ++it)
-       {
-          copyGroup(oldConfig1, *it, &cfg, *it);
-       }
-   }
-   else 
-   {
-       copyGroup(oldConfig1, oldGroup, &cfg, QString::null);
-   }
-   cfg.sync();
-   qWarning("Script: Writing entries to %s", tmp1.name().latin1());
+   if( !m_arguments.isNull())
+      log() << currentFilename << ": Running script '" << script << "' with arguments '" << m_arguments << "'" << endl;
+   else
+      log() << currentFilename << ": Running script '" << script << "'" << endl;
 
    QString cmd;
    if (interpreter.isEmpty())
@@ -543,14 +770,82 @@ void KonfUpdate::gotScript(const QString &_script)
    else
       cmd = interpreter + " " + path;
 
-   int result = system(QFile::encodeName(QString("%1 < %2 > %3").arg(cmd).arg(tmp1.name()).arg(tmp2.name())));
+   if( !m_arguments.isNull())
+   {
+      cmd += ' ';
+      cmd += m_arguments;
+   }
+
+   KTempFile tmp1;
+   tmp1.setAutoDelete(true);
+   KTempFile tmp2;
+   tmp2.setAutoDelete(true);
+   KTempFile tmp3;
+   tmp3.setAutoDelete(true);
+
+   int result;
+   if (oldConfig1)
+   {
+       if (debug)
+       {
+           tmp1.setAutoDelete(false);
+           log() << "Script input stored in " << tmp1.name() << endl;
+       }
+       KSimpleConfig cfg(tmp1.name());
+
+       if (oldGroup.isEmpty())
+       {
+           // Write all entries to tmpFile;
+           QStringList grpList = oldConfig1->groupList();
+           for(QStringList::ConstIterator it = grpList.begin();
+               it != grpList.end();
+               ++it)
+           {
+               copyGroup(oldConfig1, *it, &cfg, *it);
+           }
+       }
+       else 
+       {
+           copyGroup(oldConfig1, oldGroup, &cfg, QString::null);
+       }
+       cfg.sync();
+       result = system(QFile::encodeName(QString("%1 < %2 > %3 2> %4").arg(cmd, tmp1.name(), tmp2.name(), tmp3.name())));
+   }
+   else
+   {
+       // No config file
+       result = system(QFile::encodeName(QString("%1 2> %2").arg(cmd, tmp3.name())));
+   }
+
+   // Copy script stderr to log file
+   {
+     QFile output(tmp3.name());
+     if (output.open(IO_ReadOnly))
+     { 
+       QTextStream ts( &output );
+       ts.setEncoding(QTextStream::UnicodeUTF8);
+       while(!ts.atEnd())
+       {
+         QString line = ts.readLine();
+         log() << "[Script] " << line << endl;
+       }
+     }
+   }
+
    if (result)
    {
-      qWarning("Script: Error running '%s'", cmd.latin1());
+      log() << currentFilename << ": !! An error occured while running '" << cmd << "'" << endl;
       return;
    }
 
-   qWarning("Script: Filtered entries written to %s", tmp2.name().latin1());
+   if (!oldConfig1)
+      return; // Nothing to merge
+
+   if (debug)
+   {
+      tmp2.setAutoDelete(false);
+      log() << "Script output stored in " << tmp2.name() << endl;
+   }
 
    // Deleting old entries
    {
@@ -563,7 +858,13 @@ void KonfUpdate::gotScript(const QString &_script)
        while(!ts.atEnd())
        {
          QString line = ts.readLine();
-         if (line.startsWith("# DELETE "))
+         if (line.startsWith("["))
+         {
+            int j = line.find(']')+1;
+            if (j > 0)
+               group = line.mid(1, j-2);
+         }
+         else if (line.startsWith("# DELETE "))
          {  
             QString key = line.mid(9);
             if (key[0] == '[')
@@ -577,8 +878,10 @@ void KonfUpdate::gotScript(const QString &_script)
             }
             oldConfig2->setGroup(group);
             oldConfig2->deleteEntry(key, false);
-            if (oldConfig2->deleteGroup(group, false)) // Delete group if empty.
-               qWarning("Removing group %s", group.isEmpty() ? "<empty>" : group.latin1());
+            log() << currentFilename << ": Script removes " << oldFile << ":" << group << ":" << key << endl;
+            if (oldConfig2->deleteGroup(group, false)) { // Delete group if empty.
+               log() << currentFilename << ": Removing empty group " << oldFile << ":" << group << endl;
+	    }
          }
          else if (line.startsWith("# DELETEGROUP"))
          {
@@ -591,8 +894,9 @@ void KonfUpdate::gotScript(const QString &_script)
                   group = key.mid(1,j-2);
                }
             }
-            if (oldConfig2->deleteGroup(group, true)) // Delete group
-               qWarning("Removing group %s (FORCED)", group.isEmpty() ? "<empty>" : group.latin1());
+            if (oldConfig2->deleteGroup(group, true)) { // Delete group
+               log() << currentFilename << ": Script removes group " << oldFile << ":" << group << endl;
+	    }
           }
        }
      }
@@ -630,10 +934,11 @@ void KonfUpdate::resetOptions()
 {
    m_bCopy = false;
    m_bOverwrite = false;
+   m_arguments = QString::null;
 }
 
 
-int main(int argc, char **argv)
+extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
 {
    KAboutData aboutData("kconf_update", I18N_NOOP("KConf Update"),
                         "1.0.2",
@@ -649,4 +954,6 @@ int main(int argc, char **argv)
    KInstance instance(&aboutData);
 
    KonfUpdate konfUpdate;
+
+   return 0;
 }
