@@ -1,6 +1,6 @@
 /* vi: ts=8 sts=4 sw=4
  *
- * $Id: kdesu_pty.h,v 1.7 2003/06/07 14:01:09 mueller Exp $
+ * $Id: kdesu_pty.h,v 1.8 2004/11/15 22:43:23 mueller Exp $
  *
  * This file is part of the KDE project, module kdesu.
  * Copyright (C) 1999,2000 Geert Jansen <jansen@kde.org>
@@ -20,7 +20,9 @@
 
 #include <qcstring.h>
 
-class PTY {
+#include <kdelibs_export.h>
+
+class KDESU_EXPORT PTY {
 
 public:
     /**
