@@ -32,6 +32,7 @@
 
 #include <kaccel.h>
 #include <kaccelbase.h>
+#include <kaccelprivate.h>
 #include <kapplication.h>
 #include <kdebug.h>
 #include <kguiitem.h>
@@ -112,7 +113,7 @@ public:
 KAction::KAction( const QString& text, const KShortcut& cut,
              const QObject* receiver, const char* slot,
              KActionCollection* parent, const char* name )
-: QObject( parent, name )
+: QObject( parent, name ), d(new KActionPrivate)
 {
 	initPrivate( text, cut, receiver, slot );
 }
@@ -120,7 +121,7 @@ KAction::KAction( const QString& text, const KShortcut& cut,
 KAction::KAction( const QString& text, const QString& sIconName, const KShortcut& cut,
 	const QObject* receiver, const char* slot,
 	KActionCollection* parent, const char* name )
-: QObject( parent, name )
+: QObject( parent, name ), d(new KActionPrivate)
 {
 	initPrivate( text, cut, receiver, slot );
 	d->setIconName( sIconName );
@@ -129,7 +130,7 @@ KAction::KAction( const QString& text, const QString& sIconName, const KShortcut
 KAction::KAction( const QString& text, const QIconSet& pix, const KShortcut& cut,
 	const QObject* receiver, const char* slot,
 	KActionCollection* parent, const char* name )
-: QObject( parent, name )
+: QObject( parent, name ), d(new KActionPrivate)
 {
 	initPrivate( text, cut, receiver, slot );
 	d->setIconSet( pix );
@@ -138,19 +139,19 @@ KAction::KAction( const QString& text, const QIconSet& pix, const KShortcut& cut
 KAction::KAction( const KGuiItem& item, const KShortcut& cut,
 	const QObject* receiver, const char* slot,
 	KActionCollection* parent, const char* name )
-: QObject( parent, name )
+: QObject( parent, name ), d(new KActionPrivate)
 {
 	initPrivate( item.text(), cut, receiver, slot );
-	if( item.hasIconSet() )
+	if( item.hasIcon() )
 		setIcon( item.iconName() );
 	setToolTip( item.toolTip() );
 	setWhatsThis( item.whatsThis() );
 }
 
-// KDE 4: remove
+#ifndef KDE_NO_COMPAT // KDE 4: remove
 KAction::KAction( const QString& text, const KShortcut& cut,
                   QObject* parent, const char* name )
- : QObject( parent, name )
+ : QObject( parent, name ), d(new KActionPrivate)
 {
     initPrivate( text, cut, 0, 0 );
 }
@@ -158,7 +159,7 @@ KAction::KAction( const QString& text, const KShortcut& cut,
 KAction::KAction( const QString& text, const KShortcut& cut,
                   const QObject* receiver,
                   const char* slot, QObject* parent, const char* name )
- : QObject( parent, name )
+ : QObject( parent, name ), d(new KActionPrivate)
 {
     initPrivate( text, cut, receiver, slot );
 }
@@ -166,7 +167,7 @@ KAction::KAction( const QString& text, const KShortcut& cut,
 KAction::KAction( const QString& text, const QIconSet& pix,
                   const KShortcut& cut,
                   QObject* parent, const char* name )
- : QObject( parent, name )
+ : QObject( parent, name ), d(new KActionPrivate)
 {
     initPrivate( text, cut, 0, 0 );
     setIconSet( pix );
@@ -175,7 +176,7 @@ KAction::KAction( const QString& text, const QIconSet& pix,
 KAction::KAction( const QString& text, const QString& pix,
                   const KShortcut& cut,
                   QObject* parent, const char* name )
-: QObject( parent, name )
+: QObject( parent, name ), d(new KActionPrivate)
 {
     initPrivate( text, cut, 0, 0 );
     d->setIconName( pix );
@@ -185,7 +186,7 @@ KAction::KAction( const QString& text, const QIconSet& pix,
                   const KShortcut& cut,
                   const QObject* receiver, const char* slot, QObject* parent,
                   const char* name )
- : QObject( parent, name )
+ : QObject( parent, name ), d(new KActionPrivate)
 {
     initPrivate( text, cut, receiver, slot );
     setIconSet( pix );
@@ -195,18 +196,18 @@ KAction::KAction( const QString& text, const QString& pix,
                   const KShortcut& cut,
                   const QObject* receiver, const char* slot, QObject* parent,
                   const char* name )
-  : QObject( parent, name )
+  : QObject( parent, name ), d(new KActionPrivate)
 {
     initPrivate( text, cut, receiver, slot );
     d->setIconName(pix);
 }
 
 KAction::KAction( QObject* parent, const char* name )
- : QObject( parent, name )
+ : QObject( parent, name ), d(new KActionPrivate)
 {
     initPrivate( QString::null, KShortcut(), 0, 0 );
 }
-// KDE 4: remove end
+#endif // KDE 4: remove end
 
 KAction::~KAction()
 {
@@ -219,8 +220,15 @@ KAction::~KAction()
     // If actionCollection hasn't already been destructed,
     if ( m_parentCollection ) {
         m_parentCollection->take( this );
-        for( uint i = 0; i < d->m_kaccelList.count(); i++ )
-            d->m_kaccelList[i]->remove( name() );
+
+        const QValueList<KAccel*> & accelList = d->m_kaccelList;
+        QValueList<KAccel*>::const_iterator itr = accelList.constBegin();
+        const QValueList<KAccel*>::const_iterator itrEnd = accelList.constEnd();
+
+        const char * const namePtr = name();
+        for (; itr != itrEnd; ++itr )
+            (*itr)->remove(namePtr);
+
     }
 
     // Do not call unplugAll from here, as tempting as it sounds.
@@ -230,14 +238,12 @@ KAction::~KAction()
     // want to destroy everything asap, not to remove actions one by one
     // from the GUI.
 
-    delete d; d = 0;
+    delete d;
 }
 
 void KAction::initPrivate( const QString& text, const KShortcut& cut,
                   const QObject* receiver, const char* slot )
 {
-    d = new KActionPrivate;
-
     d->m_cutDefault = cut;
 
     m_parentCollection = dynamic_cast<KActionCollection *>( parent() );
@@ -248,7 +254,7 @@ void KAction::initPrivate( const QString& text, const KShortcut& cut,
     if ( receiver && slot )
         connect( this, SIGNAL( activated() ), receiver, slot );
 
-    if( !cut.isNull() && qstrcmp( name(), "unnamed" ) == 0 )
+    if( !cut.isNull() && !qstrcmp( name(), "unnamed" ) )
         kdWarning(129) << "KAction::initPrivate(): trying to assign a shortcut (" << cut.toStringInternal() << ") to an unnamed action." << endl;
     d->setText( text );
     initShortcut( cut );
@@ -256,7 +262,7 @@ void KAction::initPrivate( const QString& text, const KShortcut& cut,
 
 bool KAction::isPlugged() const
 {
-  return (containerCount() > 0) || d->m_kaccel;
+  return (!d->m_containers.empty()) || d->m_kaccel;
 }
 
 bool KAction::isPlugged( const QWidget *container ) const
@@ -363,7 +369,7 @@ bool KAction::initShortcut( const KShortcut& cut )
     d->m_cut = cut;
 
     // Only insert action into KAccel if it has a valid name,
-    if( qstrcmp( name(), "unnamed" ) != 0 &&
+    if( qstrcmp( name(), "unnamed" ) &&
         m_parentCollection &&
         m_parentCollection->isAutoConnectShortcuts() &&
         m_parentCollection->kaccel() )
@@ -377,13 +383,17 @@ bool KAction::initShortcut( const KShortcut& cut )
 // Only to be called from plug()
 void KAction::plugShortcut()
 {
-  KAccel* kaccel = kaccelCurrent();
+  KAccel* const kaccel = kaccelCurrent();
 
   //kdDebug(129) << "KAction::plugShortcut(): this = " << this << " kaccel() = " << (m_parentCollection ? m_parentCollection->kaccel() : 0) << endl;
-  if( kaccel && qstrcmp( name(), "unnamed" ) != 0 ) {
+  if( kaccel && qstrcmp( name(), "unnamed" ) ) {
     // Check if already plugged into current KAccel object
-    for( uint i = 0; i < d->m_kaccelList.count(); i++ ) {
-      if( d->m_kaccelList[i] == kaccel )
+    const QValueList<KAccel*> & accelList = d->m_kaccelList;
+    QValueList<KAccel*>::const_iterator itr = accelList.constBegin();
+    const QValueList<KAccel*>::const_iterator itrEnd = accelList.constEnd();
+
+    for( ; itr != itrEnd; ++itr) {
+      if( (*itr) == kaccel )
         return;
     }
 
@@ -396,16 +406,21 @@ bool KAction::setShortcut( const KShortcut& cut )
   bool bChanged = (d->m_cut != cut);
   d->m_cut = cut;
 
-  KAccel* kaccel = kaccelCurrent();
+  KAccel* const kaccel = kaccelCurrent();
   bool bInsertRequired = true;
   // Apply new shortcut to all existing KAccel objects
-  for( uint i = 0; i < d->m_kaccelList.count(); i++ ) {
+
+  const QValueList<KAccel*> & accelList = d->m_kaccelList;
+  QValueList<KAccel*>::const_iterator itr = accelList.constBegin();
+  const QValueList<KAccel*>::const_iterator itrEnd = accelList.constEnd();
+
+  for( ; itr != itrEnd; ++itr) {
     // Check whether shortcut has already been plugged into
     //  the current kaccel object.
-    if( d->m_kaccelList[i] == kaccel )
+    if( (*itr) == kaccel )
       bInsertRequired = false;
     if( bChanged )
-      updateKAccelShortcut( d->m_kaccelList[i] );
+      updateKAccelShortcut( *itr );
   }
 
   // Only insert action into KAccel if it has a valid name,
@@ -413,10 +428,10 @@ bool KAction::setShortcut( const KShortcut& cut )
     insertKAccel( kaccel );
 
   if( bChanged ) {
-    // KDE 4: remove
+#ifndef KDE_NO_COMPAT    // KDE 4: remove
     if ( d->m_kaccel )
       d->m_kaccel->setShortcut( name(), cut );
-    // KDE 4: remove end
+#endif    // KDE 4: remove end
       int len = containerCount();
       for( int i = 0; i < len; ++i )
           updateShortcut( i );
@@ -463,33 +478,38 @@ void KAction::insertKAccel( KAccel* kaccel )
 void KAction::removeKAccel( KAccel* kaccel )
 {
   //kdDebug(129) << "KAction::removeKAccel( " << i << " ): this = " << this << endl;
-  for( uint i = 0; i < d->m_kaccelList.count(); i++ ) {
-    if( d->m_kaccelList[i] == kaccel ) {
+  QValueList<KAccel*> & accelList = d->m_kaccelList;
+  QValueList<KAccel*>::iterator itr = accelList.begin();
+  const QValueList<KAccel*>::iterator itrEnd = accelList.end();
+
+  for( ; itr != itrEnd; ++itr) {
+    if( (*itr) == kaccel ) {
       kaccel->remove( name() );
-      d->m_kaccelList.remove( d->m_kaccelList.at( i ) );
+      accelList.remove( itr );
       disconnect( kaccel, SIGNAL(destroyed()), this, SLOT(slotDestroyed()) );
       break;
     }
   }
 }
 
+#ifndef KDE_NO_COMPAT
 // KDE 4: remove
 void KAction::setAccel( int keyQt )
 {
   setShortcut( KShortcut(keyQt) );
 }
-// KDE 4: remove end
+#endif // KDE 4: remove end
 
 void KAction::updateShortcut( int i )
 {
   int id = itemId( i );
 
   QWidget* w = container( i );
-  if ( w->inherits( "QPopupMenu" ) ) {
+  if ( ::qt_cast<QPopupMenu *>( w ) ) {
     QPopupMenu* menu = static_cast<QPopupMenu*>(w);
     updateShortcut( menu, id );
   }
-  else if ( w->inherits( "QMenuBar" ) )
+  else if ( ::qt_cast<QMenuBar *>( w ) )
     static_cast<QMenuBar*>(w)->setAccel( d->m_cut.keyCodeQt(), id );
 }
 
@@ -513,7 +533,7 @@ void KAction::updateShortcut( QPopupMenu* menu, int id )
     // This is a fall-hack in case the KAction is missing a proper parent collection.
     //  It should be removed eventually. --ellis
     menu->setAccel( d->m_cut.keyCodeQt(), id );
-    kdWarning(129) << "KAction::updateShortcut(): name = \"" << name() << "\", cut = " << d->m_cut.toStringInternal() << "; No KAccel, probably missing a parent collection." << endl;
+    kdDebug(129) << "KAction::updateShortcut(): name = \"" << name() << "\", cut = " << d->m_cut.toStringInternal() << "; No KAccel, probably missing a parent collection." << endl;
   }
 }
 
@@ -537,10 +557,12 @@ void KAction::setShortcutText( const QString& s )
   setShortcut( KShortcut(s) );
 }
 
+#ifndef KDE_NO_COMPAT // Remove in KDE 4
 int KAction::accel() const
 {
   return d->m_cut.keyCodeQt();
 }
+#endif
 
 void KAction::setGroup( const QString& grp )
 {
@@ -584,7 +606,7 @@ void KAction::updateToolTip( int i )
 {
   QWidget *w = container( i );
 
-  if ( w->inherits( "KToolBar" ) )
+  if ( ::qt_cast<KToolBar *>( w ) )
     QToolTip::add( static_cast<KToolBar*>(w)->getWidget( itemId( i ) ), d->toolTip() );
 }
 
@@ -596,17 +618,18 @@ QString KAction::toolTip() const
 int KAction::plug( QWidget *w, int index )
 {
   //kdDebug(129) << "KAction::plug( " << w << ", " << index << " )" << endl;
-  if (w == 0) {
+  if (!w ) {
 	kdWarning(129) << "KAction::plug called with 0 argument\n";
  	return -1;
   }
 
-#ifndef NDEBUG
+  // Ellis: print warning if there is a shortcut, but no KAccel available (often due to no widget available in the actioncollection)
+  // David: Well, it doesn't matter much, things still work (e.g. Undo in koffice) via QAccel.
+  // We should probably re-enable the warning for things that only KAccel can do, though - e.g. WIN key (mapped to Meta).
+#if 0 //ndef NDEBUG
   KAccel* kaccel = kaccelCurrent();
-  // If there is a shortcut, but no KAccel available
-  if( !d->m_cut.isNull() && kaccel == 0 ) {
-    kdWarning(129) << "KAction::plug(): has no KAccel object; this = " << this << " name = " << name() << " parentCollection = " << m_parentCollection << endl; // ellis
-    kdDebug(129) << kdBacktrace() << endl;
+  if( !d->m_cut.isNull() && !kaccel ) {
+    kdDebug(129) << "KAction::plug(): has no KAccel object; this = " << this << " name = " << name() << " parentCollection = " << m_parentCollection << endl; // ellis
   }
 #endif
 
@@ -616,7 +639,7 @@ int KAction::plug( QWidget *w, int index )
 
   plugShortcut();
 
-  if ( w->inherits("QPopupMenu") )
+  if ( ::qt_cast<QPopupMenu *>( w ) )
   {
     QPopupMenu* menu = static_cast<QPopupMenu*>( w );
     int id;
@@ -631,12 +654,12 @@ int KAction::plug( QWidget *w, int index )
         else
           instance = KGlobal::instance();
         id = menu->insertItem( d->iconSet( KIcon::Small, 0, instance ), d->text(), this,//dsweet
-                                 SLOT( slotActivated() ), keyQt,
+                                 SLOT( slotPopupActivated() ), keyQt,
                                  -1, index );
     }
     else
         id = menu->insertItem( d->text(), this,
-                               SLOT( slotActivated() ),  //dsweet
+                               SLOT( slotPopupActivated() ),
                                keyQt, -1, index );
 
     // If the shortcut is already in a KAccel object, then
@@ -660,7 +683,7 @@ int KAction::plug( QWidget *w, int index )
 
     return d->m_containers.count() - 1;
   }
-  else if ( w->inherits( "KToolBar" ) )
+  else if ( ::qt_cast<KToolBar *>( w ) )
   {
     KToolBar *bar = static_cast<KToolBar *>( w );
 
@@ -673,8 +696,8 @@ int KAction::plug( QWidget *w, int index )
 
     if ( icon().isEmpty() && !iconSet().pixmap().isNull() ) // old code using QIconSet directly
     {
-        bar->insertButton( iconSet().pixmap(), id_, SIGNAL( clicked() ), this,
-                           SLOT( slotActivated() ),
+        bar->insertButton( iconSet().pixmap(), id_, SIGNAL( buttonClicked(int, Qt::ButtonState) ), this,
+                           SLOT( slotButtonClicked(int, Qt::ButtonState) ),
                            d->isEnabled(), d->plainText(), index );
     }
     else
@@ -682,11 +705,13 @@ int KAction::plug( QWidget *w, int index )
         QString icon = d->iconName();
         if ( icon.isEmpty() )
             icon = "unknown";
-        bar->insertButton( icon, id_, SIGNAL( clicked() ), this,
-                           SLOT( slotActivated() ),
+        bar->insertButton( icon, id_, SIGNAL( buttonClicked(int, Qt::ButtonState) ), this,
+                           SLOT( slotButtonClicked(int, Qt::ButtonState) ),
                            d->isEnabled(), d->plainText(), index, instance );
     }
-    bar->getButton( id_ )->setName( QCString("toolbutton_")+name() );
+
+    KToolBarButton* ktb = bar->getButton(id_);
+    ktb->setName( QCString("toolbutton_")+name() );
 
     if ( !d->whatsThis().isEmpty() )
         QWhatsThis::add( bar->getButton(id_), whatsThisWithIcon() );
@@ -714,17 +739,17 @@ void KAction::unplug( QWidget *w )
     return;
   int id = itemId( i );
 
-  if ( w->inherits( "QPopupMenu" ) )
+  if ( ::qt_cast<QPopupMenu *>( w ) )
   {
     QPopupMenu *menu = static_cast<QPopupMenu *>( w );
     menu->removeItem( id );
   }
-  else if ( w->inherits( "KToolBar" ) )
+  else if ( ::qt_cast<KToolBar *>( w ) )
   {
     KToolBar *bar = static_cast<KToolBar *>( w );
     bar->removeItemDelayed( id );
   }
-  else if ( w->inherits( "QMenuBar" ) )
+  else if ( ::qt_cast<QMenuBar *>( w ) )
   {
     QMenuBar *bar = static_cast<QMenuBar *>( w );
     bar->removeItem( id );
@@ -794,13 +819,20 @@ void KAction::setEnabled(bool enable)
   if ( enable == d->isEnabled() )
     return;
 
+#ifndef KDE_NO_COMPAT
   // KDE 4: remove
   if (d->m_kaccel)
     d->m_kaccel->setEnabled(name(), enable);
-  // KDE 4: remove end
+#endif  // KDE 4: remove end
 
-  for ( uint i = 0; i < d->m_kaccelList.count(); i++ )
-    d->m_kaccelList[i]->setEnabled( name(), enable );
+  const QValueList<KAccel*> & accelList = d->m_kaccelList;
+  QValueList<KAccel*>::const_iterator itr = accelList.constBegin();
+  const QValueList<KAccel*>::const_iterator itrEnd = accelList.constEnd();
+
+  const char * const namePtr = name();
+
+  for ( ; itr != itrEnd; ++itr )
+    (*itr)->setEnabled( namePtr, enable );
 
   d->setEnabled( enable );
 
@@ -815,11 +847,11 @@ void KAction::updateEnabled( int i )
 {
     QWidget *w = container( i );
 
-    if ( w->inherits("QPopupMenu") )
+    if ( ::qt_cast<QPopupMenu *>( w ) )
       static_cast<QPopupMenu*>(w)->setItemEnabled( itemId( i ), d->isEnabled() );
-    else if ( w->inherits("QMenuBar") )
+    else if ( ::qt_cast<QMenuBar *>( w ) )
       static_cast<QMenuBar*>(w)->setItemEnabled( itemId( i ), d->isEnabled() );
-    else if ( w->inherits( "KToolBar" ) )
+    else if ( ::qt_cast<KToolBar *>( w ) )
       static_cast<KToolBar*>(w)->setItemEnabled( itemId( i ), d->isEnabled() );
 }
 
@@ -830,16 +862,22 @@ void KAction::setShortcutConfigurable( bool b )
 
 void KAction::setText( const QString& text )
 {
+#ifndef KDE_NO_COMPAT
   // KDE 4: remove
   if (d->m_kaccel) {
     KAccelAction* pAction = d->m_kaccel->actions().actionPtr(name());
     if (pAction)
       pAction->setLabel( text );
   }
-  // KDE 4: remove end
+#endif  // KDE 4: remove end
+  const QValueList<KAccel*> & accelList = d->m_kaccelList;
+  QValueList<KAccel*>::const_iterator itr = accelList.constBegin();
+  const QValueList<KAccel*>::const_iterator itrEnd = accelList.constEnd();
 
-  for( uint i = 0; i < d->m_kaccelList.count(); i++ ) {
-    KAccelAction* pAction = d->m_kaccelList[i]->actions().actionPtr(name());
+  const char * const namePtr = name();
+
+  for( ; itr != itrEnd; ++itr ) {
+    KAccelAction* const pAction = (*itr)->actions().actionPtr(namePtr);
     if (pAction)
       pAction->setLabel( text );
   }
@@ -855,18 +893,18 @@ void KAction::updateText( int i )
 {
   QWidget *w = container( i );
 
-  if ( w->inherits( "QPopupMenu" ) ) {
+  if ( ::qt_cast<QPopupMenu *>( w ) ) {
     int id = itemId( i );
     static_cast<QPopupMenu*>(w)->changeItem( id, d->text() );
     if (!d->m_cut.isNull())
       updateShortcut( static_cast<QPopupMenu*>(w), id );
   }
-  else if ( w->inherits( "QMenuBar" ) )
+  else if ( ::qt_cast<QMenuBar *>( w ) )
     static_cast<QMenuBar*>(w)->changeItem( itemId( i ), d->text() );
-  else if ( w->inherits( "KToolBar" ) )
+  else if ( ::qt_cast<KToolBar *>( w ) )
   {
     QWidget *button = static_cast<KToolBar *>(w)->getWidget( itemId( i ) );
-    if ( button->inherits( "KToolBarButton" ) )
+    if ( ::qt_cast<KToolBarButton *>( button ) )
       static_cast<KToolBarButton *>(button)->setText( d->plainText() );
   }
 }
@@ -895,15 +933,15 @@ void KAction::updateIcon( int id )
 {
   QWidget* w = container( id );
 
-  if ( w->inherits( "QPopupMenu" ) ) {
+  if ( ::qt_cast<QPopupMenu *>( w ) ) {
     int itemId_ = itemId( id );
     static_cast<QPopupMenu*>(w)->changeItem( itemId_, d->iconSet( KIcon::Small ), d->text() );
     if (!d->m_cut.isNull())
       updateShortcut( static_cast<QPopupMenu*>(w), itemId_ );
   }
-  else if ( w->inherits( "QMenuBar" ) )
+  else if ( ::qt_cast<QMenuBar *>( w ) )
     static_cast<QMenuBar*>(w)->changeItem( itemId( id ), d->iconSet( KIcon::Small ), d->text() );
-  else if ( w->inherits( "KToolBar" ) )
+  else if ( ::qt_cast<KToolBar *>( w ) )
     static_cast<KToolBar *>(w)->setButtonIcon( itemId( id ), d->iconName() );
 }
 
@@ -926,18 +964,18 @@ void KAction::updateIconSet( int id )
 {
   QWidget *w = container( id );
 
-  if ( w->inherits( "QPopupMenu" ) )
+  if ( ::qt_cast<QPopupMenu *>( w ) )
   {
     int itemId_ = itemId( id );
     static_cast<QPopupMenu*>(w)->changeItem( itemId_, d->iconSet(), d->text() );
     if (!d->m_cut.isNull())
       updateShortcut( static_cast<QPopupMenu*>(w), itemId_ );
   }
-  else if ( w->inherits( "QMenuBar" ) )
+  else if ( ::qt_cast<QMenuBar *>( w ) )
     static_cast<QMenuBar*>(w)->changeItem( itemId( id ), d->iconSet(), d->text() );
-  else if ( w->inherits( "KToolBar" ) )
+  else if ( ::qt_cast<KToolBar *>( w ) )
   {
-    if ( icon().isEmpty() && d->hasIconSet() ) // only if there is no named icon ( scales better )
+    if ( icon().isEmpty() && d->hasIcon() ) // only if there is no named icon ( scales better )
       static_cast<KToolBar *>(w)->setButtonIconSet( itemId( id ), d->iconSet() );
     else
       static_cast<KToolBar *>(w)->setButtonIconSet( itemId( id ), d->iconSet( KIcon::Small ) );
@@ -1049,33 +1087,89 @@ void KAction::addContainer( QWidget* c, QWidget* w )
 
 void KAction::activate()
 {
+  emit activated( KAction::EmulatedActivation, Qt::NoButton );
   slotActivated();
 }
 
 void KAction::slotActivated()
 {
+  const QObject *senderObj = sender();
+  if ( senderObj )
+  {
+    if ( ::qt_cast<KAccelPrivate *>( senderObj ) )
+        emit activated( KAction::AccelActivation, Qt::NoButton );
+  }
   emit activated();
 }
 
+// This catches signals emitted by KActions inserted into QPopupMenu
+// We do crude things inside it, because we need to know which
+// QPopupMenu emitted the signal. We need to be sure that it is
+// only called by QPopupMenus, we plugged us in.
+void KAction::slotPopupActivated()
+{
+  if( ::qt_cast<QSignal *>(sender()))
+  {
+    int id = dynamic_cast<const QSignal *>(sender())->value().toInt();
+    int pos = findContainer(id);
+    if(pos != -1)
+    {
+      QPopupMenu* qpm = dynamic_cast<QPopupMenu *>( container(pos) );
+      if(qpm)
+      {
+        KPopupMenu* kpm = dynamic_cast<KPopupMenu *>( qpm );
+        Qt::ButtonState state;
+        if ( kpm ) // KPopupMenu? Nice, it stores the state.
+            state = kpm->state();
+        else { // just a QPopupMenu? We'll ask for the state now then (small race condition?)
+            kdDebug(129) << "KAction::slotPopupActivated not a KPopupMenu -> using keyboardMouseState()" << endl;
+            state = KApplication::keyboardMouseState();
+        }
+        emit activated( KAction::PopupMenuActivation, state );
+        slotActivated();
+        return;
+      }
+    }
+  }
+
+  kdWarning(129)<<"Don't connect KAction::slotPopupActivated() to anything, expect into QPopupMenus which are in containers. Use slotActivated instead."<<endl;
+  emit activated( KAction::PopupMenuActivation, Qt::NoButton );
+  slotActivated();
+}
+
+void KAction::slotButtonClicked( int, Qt::ButtonState state )
+{
+  kdDebug(129) << "slotButtonClicked() state=" << state << endl;
+  emit activated( KAction::ToolBarActivation, state );
+
+  // RightButton isn't really an activation
+  if ( ( state & LeftButton ) || ( state & MidButton ) )
+    slotActivated();
+}
+
+
 void KAction::slotDestroyed()
 {
   kdDebug(129) << "KAction::slotDestroyed(): this = " << this << ", name = \"" << name() << "\", sender = " << sender() << endl;
-  const QObject* o = sender();
+  const QObject* const o = sender();
 
-  // KDE 4: remove
+#ifndef KDE_NO_COMPAT  // KDE 4: remove
   if ( o == d->m_kaccel )
   {
     d->m_kaccel = 0;
     return;
   }
-  // KDE 4: remove end
+#endif  // KDE 4: remove end
+  QValueList<KAccel*> & accelList = d->m_kaccelList;
+  QValueList<KAccel*>::iterator itr = accelList.begin();
+  const QValueList<KAccel*>::iterator itrEnd = accelList.end();
 
-  for( uint i = 0; i < d->m_kaccelList.count(); i++ )
+  for( ; itr != itrEnd; ++itr)
   {
-    if ( o == d->m_kaccelList[i] )
+    if ( o == *itr )
     {
-      disconnect( d->m_kaccelList[i], SIGNAL(destroyed()), this, SLOT(slotDestroyed()) );
-      d->m_kaccelList.remove( d->m_kaccelList.at( i ) );
+      disconnect( *itr, SIGNAL(destroyed()), this, SLOT(slotDestroyed()) );
+      accelList.remove(itr);
       return;
     }
   }
@@ -1092,8 +1186,13 @@ void KAction::slotDestroyed()
 int KAction::findContainer( const QWidget* widget ) const
 {
   int pos = 0;
-  QValueList<KActionPrivate::Container>::ConstIterator it = d->m_containers.begin();
-  while( it != d->m_containers.end() )
+
+  const QValueList<KActionPrivate::Container> & containers = d->m_containers;
+
+  QValueList<KActionPrivate::Container>::ConstIterator it = containers.constBegin();
+  const QValueList<KActionPrivate::Container>::ConstIterator itEnd = containers.constEnd();
+
+  while( it != itEnd )
   {
     if ( (*it).m_representative == widget || (*it).m_container == widget )
       return pos;
@@ -1104,15 +1203,40 @@ int KAction::findContainer( const QWidget* widget ) const
   return -1;
 }
 
+int KAction::findContainer( const int id ) const
+{
+  int pos = 0;
+
+  const QValueList<KActionPrivate::Container> & containers = d->m_containers;
+
+  QValueList<KActionPrivate::Container>::ConstIterator it = containers.constBegin();
+  const QValueList<KActionPrivate::Container>::ConstIterator itEnd = containers.constEnd();
+
+  while( it != itEnd )
+  {
+    if ( (*it).m_id == id )
+      return pos;
+    ++it;
+    ++pos;
+  }
+
+  return -1;
+}
+
 void KAction::removeContainer( int index )
 {
   int i = 0;
-  QValueList<KActionPrivate::Container>::Iterator it = d->m_containers.begin();
-  while( it != d->m_containers.end() )
+
+  QValueList<KActionPrivate::Container> & containers = d->m_containers;
+
+  QValueList<KActionPrivate::Container>::Iterator it = containers.begin();
+  const QValueList<KActionPrivate::Container>::Iterator itEnd = containers.end();
+
+  while( it != itEnd )
   {
     if ( i == index )
     {
-      d->m_containers.remove( it );
+      containers.remove( it );
       return;
     }
     ++it;
@@ -1140,6 +1264,11 @@ void KAction::unplugAll()
     unplug( container( 0 ) );
 }
 
+const KGuiItem& KAction::guiItem() const
+{
+    return *d;
+}
+
 void KAction::virtual_hook( int, void* )
 { /*BASE::virtual_hook( id, data );*/ }
 
