@@ -147,14 +147,14 @@ int KSyntaxHighlighter::highlightParagraph( const QString &text, int )
     }
 
     QString simplified = text;
-    simplified = simplified.replace( QRegExp( "\\s" ), "" ).replace( "|", ">" );
-    while ( simplified.startsWith( ">>>>" ) )
+    simplified = simplified.replace( QRegExp( "\\s" ), QString::null ).replace( '|', QString::fromLatin1(">") );
+    while ( simplified.startsWith( QString::fromLatin1(">>>>") ) )
 	simplified = simplified.mid(3);
-    if	( simplified.startsWith( ">>>" ) || simplified.startsWith( "> >	>" ) )
+    if	( simplified.startsWith( QString::fromLatin1(">>>") ) || simplified.startsWith( QString::fromLatin1("> >	>") ) )
 	setFormat( 0, text.length(), d->col2 );
-    else if	( simplified.startsWith( ">>" )	|| simplified.startsWith( "> >"	) )
+    else if	( simplified.startsWith( QString::fromLatin1(">>") ) || simplified.startsWith( QString::fromLatin1("> >") ) )
 	setFormat( 0, text.length(), d->col3 );
-    else if	( simplified.startsWith( ">" ) )
+    else if	( simplified.startsWith( QString::fromLatin1(">") ) )
 	setFormat( 0, text.length(), d->col4 );
     else
 	setFormat( 0, text.length(), d->col5 );
@@ -282,7 +282,7 @@ KDictSpellingHighlighter::KDictSpellingHighlighter( QTextEdit *textEdit,
     d = new KDictSpellingHighlighterPrivate();
 
     d->mSpellConfig = spellConfig;
-    d->globalConfig = ( spellConfig == 0 );
+    d->globalConfig = ( !spellConfig );
     d->automatic = autoEnable;
     d->active = spellCheckingActive;
     d->checksRequested = 0;
@@ -389,7 +389,7 @@ bool KDictSpellingHighlighter::isMisspelled( const QString &word )
 	return false;
     }
 
-    if ((dict->isEmpty() || ((*dict)[word] == 0)) && d->spell ) {
+    if ((dict->isEmpty() || !((*dict)[word])) && d->spell ) {
 	int para, index;
 	textEdit()->getCursorPosition( &para, &index );
 	++d->wordCount;
