@@ -139,23 +139,23 @@ void HTMLBodyElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     }
     case ATTR_ONLOAD:
-        getDocument()->setWindowEventListener(EventImpl::LOAD_EVENT,
+        getDocument()->setHTMLWindowEventListener(EventImpl::LOAD_EVENT,
 	    getDocument()->createHTMLEventListener(attr->value().string(),"onload"));
         break;
     case ATTR_ONUNLOAD:
-        getDocument()->setWindowEventListener(EventImpl::UNLOAD_EVENT,
+        getDocument()->setHTMLWindowEventListener(EventImpl::UNLOAD_EVENT,
 	    getDocument()->createHTMLEventListener(attr->value().string(),"onunload"));
         break;
     case ATTR_ONBLUR:
-        getDocument()->setWindowEventListener(EventImpl::BLUR_EVENT,
+        getDocument()->setHTMLWindowEventListener(EventImpl::BLUR_EVENT,
 	    getDocument()->createHTMLEventListener(attr->value().string(),"onblur"));
         break;
     case ATTR_ONFOCUS:
-        getDocument()->setWindowEventListener(EventImpl::FOCUS_EVENT,
+        getDocument()->setHTMLWindowEventListener(EventImpl::FOCUS_EVENT,
 	    getDocument()->createHTMLEventListener(attr->value().string(),"onfocus"));
         break;
     case ATTR_ONRESIZE:
-        getDocument()->setWindowEventListener(EventImpl::RESIZE_EVENT,
+        getDocument()->setHTMLWindowEventListener(EventImpl::RESIZE_EVENT,
 	    getDocument()->createHTMLEventListener(attr->value().string(),"onresize"));
         break;
     case ATTR_NOSAVE:
@@ -280,13 +280,11 @@ void HTMLFrameElementImpl::parseAttribute(AttributeImpl *attr)
         // when attached, has no effect
         break;
     case ATTR_ONLOAD:
-        static_cast<HTMLDocumentImpl*>( getDocument() )->body()
-              ->setHTMLEventListener(EventImpl::LOAD_EVENT,
+        setHTMLEventListener(EventImpl::LOAD_EVENT,
             getDocument()->createHTMLEventListener(attr->value().string(),"onload"));
         break;
     case ATTR_ONUNLOAD:
-        static_cast<HTMLDocumentImpl*>( getDocument() )->body()
-              ->setHTMLEventListener(EventImpl::UNLOAD_EVENT,
+        setHTMLEventListener(EventImpl::UNLOAD_EVENT,
             getDocument()->createHTMLEventListener(attr->value().string(),"onunload"));
         break;
 
@@ -378,7 +376,7 @@ void HTMLFrameElementImpl::setLocation( const DOMString& str )
     }
 }
 
-bool HTMLFrameElementImpl::isSelectable() const
+bool HTMLFrameElementImpl::isFocusable() const
 {
     return m_render!=0;
 }
@@ -532,7 +530,7 @@ void HTMLFrameSetElementImpl::detach()
 void HTMLFrameSetElementImpl::recalcStyle( StyleChange ch )
 {
     if (changed() && m_render) {
-        m_render->setLayouted(false);
+        m_render->setNeedsLayout(true);
 //         m_render->layout();
         setChanged(false);
     }
@@ -576,21 +574,27 @@ NodeImpl::Id HTMLIFrameElementImpl::id() const
 
 void HTMLIFrameElementImpl::parseAttribute(AttributeImpl *attr )
 {
-  switch (  attr->id() )
-  {
+    switch (  attr->id() )
+    {
     case ATTR_WIDTH:
-      addCSSLength( CSS_PROP_WIDTH, attr->value());
-      break;
+        if (!attr->value().isEmpty())
+            addCSSLength(CSS_PROP_WIDTH, attr->value());
+        else
+            removeCSSProperty(CSS_PROP_WIDTH);
+        break;
     case ATTR_HEIGHT:
-      addCSSLength( CSS_PROP_HEIGHT, attr->value() );
-      break;
+        if (!attr->value().isEmpty())
+            addCSSLength(CSS_PROP_HEIGHT, attr->value());
+        else
+            removeCSSProperty(CSS_PROP_HEIGHT);
+        break;
     case ATTR_SRC:
-      needWidgetUpdate = true; // ### do this for scrolling, margins etc?
-      HTMLFrameElementImpl::parseAttribute( attr );
-      break;
+        needWidgetUpdate = true; // ### do this for scrolling, margins etc?
+        HTMLFrameElementImpl::parseAttribute( attr );
+        break;
     default:
-      HTMLFrameElementImpl::parseAttribute( attr );
-  }
+        HTMLFrameElementImpl::parseAttribute( attr );
+    }
 }
 
 void HTMLIFrameElementImpl::attach()
