@@ -79,10 +79,16 @@ void HTMLImageElementImpl::parseAttribute(AttributeImpl *attr)
         setChanged();
         break;
     case ATTR_WIDTH:
-        addCSSLength(CSS_PROP_WIDTH, attr->value());
+        if (!attr->value().isEmpty())
+            addCSSLength(CSS_PROP_WIDTH, attr->value());
+        else
+            removeCSSProperty(CSS_PROP_WIDTH);
         break;
     case ATTR_HEIGHT:
-        addCSSLength(CSS_PROP_HEIGHT, attr->value());
+        if (!attr->value().isEmpty())
+            addCSSLength(CSS_PROP_HEIGHT, attr->value());
+        else
+            removeCSSProperty(CSS_PROP_HEIGHT);
         break;
     case ATTR_BORDER:
         // border="noborder" -> border="0"
@@ -222,6 +228,15 @@ QImage HTMLImageElementImpl::currentImage() const
     return QImage();
 }
 
+QPixmap HTMLImageElementImpl::currentPixmap() const
+{
+    RenderImage *r = static_cast<RenderImage*>(renderer());
+
+    if(r)
+        return r->pixmap();
+
+    return QPixmap();
+}
 
 bool HTMLImageElementImpl::complete() const
 {
