@@ -181,7 +181,7 @@ namespace KWallet
  * @author Lars Knoll (knoll@kde.org)
  *
  */
-class KHTMLPart : public KParts::ReadOnlyPart
+class KHTML_EXPORT KHTMLPart : public KParts::ReadOnlyPart
 {
   Q_OBJECT
   friend class KHTMLView;
@@ -279,6 +279,12 @@ public:
    * Returns a reference to the DOM document.
    */
   DOM::Document document() const;
+  
+  /**
+   * Returns the content of the source document.
+   * @since 3.4
+   */
+   QString documentSource() const;
 
   /**
    * Returns the node that has the keyboard focus.
@@ -709,7 +715,7 @@ public:
   /**
    * Finds the next occurrence of the string or expression.
    * If isRegExp is true then str is converted to a QRegExp, and caseSensitive is ignored.
-   * @DEPRECATED, use findText( str, options, parent, findDialog )
+   * @deprecated, use findText( str, options, parent, findDialog )
    */
   bool findTextNext( const QString &str, bool forward, bool caseSensitive, bool isRegExp );
 
@@ -742,6 +748,16 @@ public:
    */
   virtual QString selectedText() const;
 
+  /**
+   * Return the text the user has marked.  This is guaranteed to be valid xml,
+   * and to contain the \<html> and \<body> tags.
+   *
+   * FIXME probably should make virtual for 4.0 ?
+   * 
+   * @since 3.4
+   */
+  QString selectedTextAsHTML() const;
+
   /**
    * Returns the selected part of the HTML.
    */
@@ -823,7 +839,7 @@ public:
   /**
    * Returns the @p Node currently under the mouse.
    *
-   * The returned node may be a shared node (e. g. an <area> node if the
+   * The returned node may be a shared node (e. g. an \<area> node if the
    * mouse is hovering over an image map).
    */
   DOM::Node nodeUnderMouse() const;
@@ -833,7 +849,7 @@ public:
    *
    * The returned node is always the node that is physically under the mouse
    * pointer (irrespective of logically overlying elements like, e. g.,
-   * <area> on image maps).
+   * \<area> on image maps).
    * @since 3.3
    */
   DOM::Node nonSharedNodeUnderMouse() const;
@@ -992,6 +1008,12 @@ public:
    */
   bool isModified() const;
 
+  /**
+   * Shows or hides the suppressed popup indicator
+   * @since 3.4
+   */
+  void setSuppressedPopupIndicator( bool enable );
+
 signals:
   /**
    * Emitted if the cursor is moved over an URL.
@@ -1073,11 +1095,11 @@ protected:
    */
   virtual void khtmlMouseDoubleClickEvent( khtml::MouseDoubleClickEvent * );
   /**
-   * Eventhandler for the khtml::MouseDoubleMoveEvent.
+   * Eventhandler for the khtml::MouseMouseMoveEvent.
    */
   virtual void khtmlMouseMoveEvent( khtml::MouseMoveEvent *event );
   /**
-   * Eventhandler for the khtml::MouseMoveEvent.
+   * Eventhandler for the khtml::MouseMouseReleaseEvent.
    */
   virtual void khtmlMouseReleaseEvent( khtml::MouseReleaseEvent *event );
   /**
@@ -1217,11 +1239,6 @@ public slots:
                         const QString& boundary = QString::null );
 
 private slots:
-  /**
-   * @internal
-   * Jump to the anchor as defined in the url. Typically connected to KHTMLView::finishedLayout().
-   */
-  void gotoAnchor();
 
   /**
    * @internal
@@ -1455,6 +1472,8 @@ private slots:
 
   /**
    * @internal
+   * used to restore or reset the view's scroll position (including positioning on anchors)
+   * once a sufficient portion of the document as been laid out.
    */
   void restoreScrollPosition();
 
@@ -1466,8 +1485,9 @@ private:
 
   enum StatusBarPriority { BarDefaultText, BarHoverText, BarOverrideText };
   void setStatusBarText( const QString& text, StatusBarPriority p);
-
+  
   bool restoreURL( const KURL &url );
+  void resetFromScript();
   void emitSelectionChanged();
   // Returns whether callingHtmlPart may access this part
   bool checkFrameAccess(KHTMLPart *callingHtmlPart);
@@ -1475,6 +1495,7 @@ private:
   void startAutoScroll();
   void stopAutoScroll();
   void overURL( const QString &url, const QString &target, bool shiftPressed = false );
+  void resetHoverText(); // Undo overURL and reset HoverText
 
   bool processObjectRequest( khtml::ChildFrame *child, const KURL &url, const QString &mimetype );
 
