@@ -39,7 +39,7 @@ class KURLBar;
  * @see KURLBar
  * @see KURLBarListBox
  */
-class KURLBarItem : public QListBoxPixmap
+class KIO_EXPORT KURLBarItem : public QListBoxPixmap
 {
 public:
     /**
@@ -227,7 +227,7 @@ class KURLBarListBox;
  * @author Carsten Pfeiffer <pfeiffer@kde.org>
  * @short A URL-bar widget, as used in the KFileDialog
  */
-class KURLBar : public QFrame
+class KIO_EXPORT KURLBar : public QFrame
 {
     Q_OBJECT
 
@@ -418,6 +418,8 @@ protected:
 
     virtual void resizeEvent( QResizeEvent * );
 
+    virtual void paletteChange( const QPalette & );
+
     /**
      * The currently active item.
      */
@@ -488,7 +490,7 @@ class KURLBarToolTip;
  *
  * @author Carsten Pfeiffer <pfeiffer@kde.org>
  */
-class KURLBarListBox : public KListBox
+class KIO_EXPORT KURLBarListBox : public KListBox
 {
     Q_OBJECT
 
@@ -532,6 +534,7 @@ protected:
     virtual void contentsDragEnterEvent( QDragEnterEvent * );
     virtual void contentsDropEvent( QDropEvent * );
     virtual void contextMenuEvent( QContextMenuEvent * );
+    virtual void paintEvent( QPaintEvent* );
 
 private:
     Qt::Orientation m_orientation;
@@ -561,7 +564,7 @@ class KURLRequester;
  *
  * @author Carsten Pfeiffer <pfeiffer@kde.org>
  */
-class KURLBarItemDialog : public KDialogBase
+class KIO_EXPORT KURLBarItemDialog : public KDialogBase
 {
     Q_OBJECT
 
