@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
    Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
                  2000 Carsten Pfeiffer <pfeiffer@kde.org>
-                 2001-2004 Michael Brade <brade@kde.org>
+                 2001-2005 Michael Brade <brade@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -452,7 +452,7 @@ void KDirListerCache::forgetDirs( KDirLister *lister, const KURL& _url, bool not
 
       // this job is a running update
       KIO::ListJob *job = jobForUrl(urlStr);
-      if (job)
+      if ( job )
       {
         lister->jobDone(job);
         killJob( job );
@@ -477,13 +477,33 @@ void KDirListerCache::forgetDirs( KDirLister *lister, const KURL& _url, bool not
         kdDebug(7004) << k_funcinfo << lister << " item moved into cache: " << url << endl;
         itemsCached.insert( urlStr, item ); // TODO: may return false!!
 
-        // watch cached directories if not manually mounted, otherwise set to "dirty"
-        if ( !KIO::manually_mounted( item->url.directory( false ) + item->url.fileName() ) )
-          item->incAutoUpdate();
+        // Should we forget the dir for good, or keep a watch on it?
+        // Generally keep a watch, except when it would prevent
+        // unmounting a removable device (#37780)
+        const bool isLocal = item->url.isLocalFile();
+        const bool isManuallyMounted = isLocal && KIO::manually_mounted( item->url.path() );
+        bool containsManuallyMounted = false;
+        if ( !isManuallyMounted && item->lstItems && isLocal ) {
+          // Look for a manually-mounted directory inside
+          // If there's one, we can't keep a watch either, FAM would prevent unmounting the CDROM
+          // I hope this isn't too slow (manually_mounted caches the last device so most
+          // of the time this is just a stat per subdir)
+          KFileItemListIterator kit( *item->lstItems );
+          for ( ; kit.current() && !containsManuallyMounted; ++kit )
+            if ( (*kit)->isDir() && KIO::manually_mounted( (*kit)->url().path() ) )
+              containsManuallyMounted = true;
+        }
+        if ( isManuallyMounted || containsManuallyMounted ) {
+          kdDebug(7004) << "Not adding a watch on " << item->url << " because it " <<
+            ( isManuallyMounted ? "is manually mounted" : "contains a manually mounted subdir" )
+                        << endl;
+          item->complete = false; // set to "dirty"
+        }
         else
-          item->complete = false;
+          item->incAutoUpdate(); // keep watch
       }
-      else {
+      else
+      {
         delete item;
         item = 0;
       }
@@ -509,17 +529,19 @@ void KDirListerCache::updateDirectory( const KURL& _dir )
 
   QPtrList<KDirLister> *listers = urlsCurrentlyListed[urlStr];
   QPtrList<KDirLister> *holders = urlsCurrentlyHeld[urlStr];
+
   // restart the job for _dir if it is running already
   bool killed = false;
-  KIO::ListJob *job = jobForUrl(urlStr);
-  if (job)
+  KIO::ListJob *job = jobForUrl( urlStr );
+  if ( job )
   {
      killed = true;
      killJob( job );
-     if (listers)
+     if ( listers )
         for ( KDirLister *kdl = listers->first(); kdl; kdl = listers->next() )
            kdl->jobDone(job);
-     if (holders)
+
+     if ( holders )
         for ( KDirLister *kdl = holders->first(); kdl; kdl = holders->next() )
            kdl->jobDone(job);
   }
@@ -528,35 +550,35 @@ void KDirListerCache::updateDirectory( const KURL& _dir )
   // we don't need to emit canceled signals since we only replaced the job,
   // the listing is continuing.
 
-  Q_ASSERT( !listers || ( listers && killed ) );
+  Q_ASSERT( !listers || (listers && killed) );
 
   job = KIO::listDir( _dir, false /* no default GUI */ );
   jobs.insert( job, QValueList<KIO::UDSEntry>() );
 
-  connect( job, SIGNAL( entries( KIO::Job *, const KIO::UDSEntryList & ) ),
-           this, SLOT( slotUpdateEntries( KIO::Job *, const KIO::UDSEntryList & ) ) );
-  connect( job, SIGNAL( result( KIO::Job * ) ),
-           this, SLOT( slotUpdateResult( KIO::Job * ) ) );
+  connect( job, SIGNAL(entries( KIO::Job *, const KIO::UDSEntryList & )),
+           this, SLOT(slotUpdateEntries( KIO::Job *, const KIO::UDSEntryList & )) );
+  connect( job, SIGNAL(result( KIO::Job * )),
+           this, SLOT(slotUpdateResult( KIO::Job * )) );
 
   kdDebug(7004) << k_funcinfo << "update started in " << _dir << endl;
 
-  if (listers)
+  if ( listers )
      for ( KDirLister *kdl = listers->first(); kdl; kdl = listers->next() )
-        kdl->jobStarted(job);
+        kdl->jobStarted( job );
 
-  if (holders)
+  if ( holders )
   {
      if ( killed )
      {
         bool first = true;
         for ( KDirLister *kdl = holders->first(); kdl; kdl = holders->next() )
         {
-           kdl->jobStarted(job);
+           kdl->jobStarted( job );
            kdl->d->complete = false;
-           if (first && kdl->d->window)
+           if ( first && kdl->d->window )
            {
               first = false;
-              job->setWindow(kdl->d->window);
+              job->setWindow( kdl->d->window );
            }
            emit kdl->started( _dir );
         }
@@ -564,7 +586,7 @@ void KDirListerCache::updateDirectory( const KURL& _dir )
      else
      {
         for ( KDirLister *kdl = holders->first(); kdl; kdl = holders->next() )
-           kdl->jobStarted(job);
+           kdl->jobStarted( job );
      }
   }
 }
@@ -590,7 +612,7 @@ bool KDirListerCache::checkUpdate( const QString& _dir )
     return true;
 }
 
-KFileItemList* KDirListerCache::itemsForDir( const KURL &_dir ) const
+KFileItemList *KDirListerCache::itemsForDir( const KURL &_dir ) const
 {
   QString urlStr = _dir.url(-1);
   DirItem *item = itemsInUse[ urlStr ];
@@ -599,7 +621,7 @@ KFileItemList* KDirListerCache::itemsForDir( const KURL &_dir ) const
   return item ? item->lstItems : 0;
 }
 
-KFileItem* KDirListerCache::findByName( const KDirLister *lister, const QString& _name ) const
+KFileItem *KDirListerCache::findByName( const KDirLister *lister, const QString& _name ) const
 {
   Q_ASSERT( lister );
 
@@ -615,7 +637,7 @@ KFileItem* KDirListerCache::findByName( const KDirLister *lister, const QString&
   return 0L;
 }
 
-KFileItem* KDirListerCache::findByURL( const KDirLister *lister, const KURL& _u ) const
+KFileItem *KDirListerCache::findByURL( const KDirLister *lister, const KURL& _u ) const
 {
   KURL _url = _u;
   _url.adjustPath(-1);
@@ -627,7 +649,7 @@ KFileItem* KDirListerCache::findByURL( const KDirLister *lister, const KURL& _u
   if ( lister && !lister->d->lstDirs.contains( parentDir ) )
       return 0L;
 
-  KFileItemList* itemList = itemsForDir( parentDir );
+  KFileItemList *itemList = itemsForDir( parentDir );
   if ( itemList )
   {
     KFileItemListIterator kit( *itemList );
@@ -651,13 +673,13 @@ void KDirListerCache::FilesRemoved( const KURL::List &fileList )
   for ( ; it != fileList.end() ; ++it )
   {
     // emit the deleteItem signal if this file was shown in any view
-    KFileItem* fileitem = 0L;
+    KFileItem *fileitem = 0L;
     KURL parentDir( *it );
     parentDir.setPath( parentDir.directory() );
-    KFileItemList* lstItems = itemsForDir( parentDir );
+    KFileItemList *lstItems = itemsForDir( parentDir );
     if ( lstItems )
     {
-      KFileItem* fit = lstItems->first();
+      KFileItem *fit = lstItems->first();
       for ( ; fit; fit = lstItems->next() )
         if ( fit->url() == *it ) {
           fileitem = fit;
@@ -699,10 +721,11 @@ void KDirListerCache::FilesChanged( const KURL::List &fileList )
     if ( ( *it ).isLocalFile() )
     {
       kdDebug(7004) << "KDirListerCache::FilesChanged " << *it << endl;
-      KFileItem* fileitem = findByURL( 0, *it );
+      KFileItem *fileitem = findByURL( 0, *it );
       if ( fileitem )
       {
           // we need to refresh the item, because e.g. the permissions can have changed.
+          aboutToRefreshItem( fileitem );
           fileitem->refresh();
           emitRefreshItem( fileitem );
       }
@@ -712,7 +735,7 @@ void KDirListerCache::FilesChanged( const KURL::List &fileList )
       // For remote files, refresh() won't be able to figure out the new information.
       // Let's update the dir.
       KURL dir( *it );
-      dir.setPath( dir.directory(-1) );
+      dir.setPath( dir.directory( true ) );
       if ( dirsToUpdate.find( dir ) == dirsToUpdate.end() )
         dirsToUpdate.prepend( dir );
     }
@@ -739,12 +762,12 @@ void KDirListerCache::FileRenamed( const KURL &src, const KURL &dst )
   // Now update the KFileItem representing that file or dir (not exclusive with the above!)
   KURL oldurl( src );
   oldurl.adjustPath( -1 );
-  KFileItem* fileitem = findByURL( 0, oldurl );
+  KFileItem *fileitem = findByURL( 0, oldurl );
   if ( fileitem )
   {
+    aboutToRefreshItem( fileitem );
     fileitem->setURL( dst );
     fileitem->refreshMimeType();
-
     emitRefreshItem( fileitem );
   }
 #ifdef DEBUG_CACHE
@@ -752,7 +775,25 @@ void KDirListerCache::FileRenamed( const KURL &src, const KURL &dst )
 #endif
 }
 
-void KDirListerCache::emitRefreshItem( KFileItem* fileitem )
+void KDirListerCache::aboutToRefreshItem( KFileItem *fileitem )
+{
+  // Look whether this item was shown in any view, i.e. held by any dirlister
+  KURL parentDir( fileitem->url() );
+  parentDir.setPath( parentDir.directory() );
+  QString parentDirURL = parentDir.url();
+  QPtrList<KDirLister> *listers = urlsCurrentlyHeld[parentDirURL];
+  if ( listers )
+    for ( KDirLister *kdl = listers->first(); kdl; kdl = listers->next() )
+      kdl->aboutToRefreshItem( fileitem );
+
+  // Also look in urlsCurrentlyListed, in case the user manages to rename during a listing
+  listers = urlsCurrentlyListed[parentDirURL];
+  if ( listers )
+    for ( KDirLister *kdl = listers->first(); kdl; kdl = listers->next() )
+      kdl->aboutToRefreshItem( fileitem );
+}
+
+void KDirListerCache::emitRefreshItem( KFileItem *fileitem )
 {
   // Look whether this item was shown in any view, i.e. held by any dirlister
   KURL parentDir( fileitem->url() );
@@ -827,6 +868,7 @@ void KDirListerCache::slotFileDirtyDelayed()
   if ( item )
   {
     // we need to refresh the item, because e.g. the permissions can have changed.
+    aboutToRefreshItem( item );
     item->refresh();
     emitRefreshItem( item );
   }
@@ -1057,13 +1099,14 @@ void KDirListerCache::renameDir( const KURL &oldUrl, const KURL &newUrl )
   while ( itu.current() )
   {
     goNext = true;
-    DirItem* dir = itu.current();
+    DirItem *dir = itu.current();
     KURL oldDirUrl ( itu.currentKey() );
     //kdDebug(7004) << "itemInUse: " << oldDirUrl.prettyURL() << endl;
     // Check if this dir is oldUrl, or a subfolder of it
     if ( oldUrl.isParentOf( oldDirUrl ) )
     {
-      QString relPath = oldDirUrl.path().mid( oldUrl.path().length() ); // ### should use KURL::cleanpath like isParentOf does
+      // TODO should use KURL::cleanpath like isParentOf does
+      QString relPath = oldDirUrl.path().mid( oldUrl.path().length() );
 
       KURL newDirUrl( newUrl ); // take new base
       if ( !relPath.isEmpty() )
@@ -1132,9 +1175,8 @@ void KDirListerCache::emitRedirections( const KURL &oldUrl, const KURL &url )
   if ( holders )
   {
     for ( KDirLister *kdl = holders->first(); kdl; kdl = holders->next() )
-    {
        kdl->jobDone(job);
-    }
+
     urlsCurrentlyHeld.insert( urlStr, holders );
   }
 
@@ -1144,9 +1186,7 @@ void KDirListerCache::emitRedirections( const KURL &oldUrl, const KURL &url )
 
     // Tell the world about the new url
     for ( KDirLister *kdl = listers->first(); kdl; kdl = listers->next() )
-    {
       emit kdl->started( url );
-    }
   }
 
   if (holders)
@@ -1155,10 +1195,10 @@ void KDirListerCache::emitRedirections( const KURL &oldUrl, const KURL &url )
     for ( KDirLister *kdl = holders->first(); kdl; kdl = holders->next() )
     {
       *kdl->d->lstDirs.find( oldUrl ) = url;
+
       if ( kdl->d->lstDirs.count() == 1 )
-      {
         emit kdl->redirection( url );
-      }
+
       emit kdl->redirection( oldUrl, url );
     }
   }
@@ -1184,6 +1224,7 @@ void KDirListerCache::slotUpdateEntries( KIO::Job* job, const KIO::UDSEntryList&
 
 void KDirListerCache::slotUpdateResult( KIO::Job * j )
 {
+
   Q_ASSERT( j );
   KIO::ListJob *job = static_cast<KIO::ListJob *>( j );
 
@@ -1221,6 +1262,7 @@ void KDirListerCache::slotUpdateResult( KIO::Job * j )
     for ( kdl = listers->first(); kdl; kdl = listers->next() )
     {
       kdl->jobDone(job);
+
       //don't bother the user
       //kdl->handleError( job );
 
@@ -1318,6 +1360,9 @@ void KDirListerCache::slotUpdateResult( KIO::Job * j )
       // check if something changed for this file
       if ( !tmp->cmp( *item ) )
       {
+        for ( kdl = listers->first(); kdl; kdl = listers->next() )
+          kdl->aboutToRefreshItem( tmp );
+
         //kdDebug(7004) << "slotUpdateResult: file changed: " << tmp->name() << endl;
         tmp->assign( *item );
 
@@ -1660,6 +1705,11 @@ const KURL& KDirLister::url() const
   return d->url;
 }
 
+const KURL::List& KDirLister::directories() const
+{
+  return d->lstDirs;
+}
+
 void KDirLister::emitChanges()
 {
   if ( d->changes == NONE )
@@ -1759,23 +1809,23 @@ bool KDirLister::isFinished() const
   return d->complete;
 }
 
-KFileItem* KDirLister::rootItem() const
+KFileItem *KDirLister::rootItem() const
 {
   return d->rootFileItem;
 }
 
-KFileItem* KDirLister::findByURL( const KURL& _url ) const
+KFileItem *KDirLister::findByURL( const KURL& _url ) const
 {
   return s_pCache->findByURL( this, _url );
 }
 
-KFileItem* KDirLister::findByName( const QString& _name ) const
+KFileItem *KDirLister::findByName( const QString& _name ) const
 {
   return s_pCache->findByName( this, _name );
 }
 
 #ifndef KDE_NO_COMPAT
-KFileItem* KDirLister::find( const KURL& _url ) const
+KFileItem *KDirLister::find( const KURL& _url ) const
 {
   return findByURL( _url );
 }
@@ -1815,9 +1865,9 @@ void KDirLister::setMimeFilter( const QStringList& mimeFilter )
   if ( !(d->changes & MIME_FILTER) )
     d->oldMimeFilter = d->mimeFilter;
 
-  if (mimeFilter.find ("all/allfiles") != mimeFilter.end () ||
-      mimeFilter.find ("all/all") != mimeFilter.end ())
-    d->mimeFilter.clear ();
+  if ( mimeFilter.find("all/allfiles") != mimeFilter.end() || 
+       mimeFilter.find("all/all") != mimeFilter.end() )
+    d->mimeFilter.clear();
   else
     d->mimeFilter = mimeFilter;
 
@@ -1838,8 +1888,8 @@ void KDirLister::clearMimeFilter()
 {
   if ( !(d->changes & MIME_FILTER) )
   {
-       d->oldMimeFilter = d->mimeFilter;
-       d->oldMimeExcludeFilter = d->mimeExcludeFilter;
+    d->oldMimeFilter = d->mimeFilter;
+    d->oldMimeExcludeFilter = d->mimeExcludeFilter;
   }
   d->mimeFilter.clear();
   d->mimeExcludeFilter.clear();
@@ -1883,6 +1933,9 @@ bool KDirLister::matchesFilter( const KFileItem *item ) const
 bool KDirLister::matchesMimeFilter( const KFileItem *item ) const
 {
   Q_ASSERT( item );
+  // Don't lose time determining the mimetype if there is no filter
+  if ( d->mimeFilter.isEmpty() && d->mimeExcludeFilter.isEmpty() )
+      return true;
   return matchesMimeFilter( item->mimetype() );
 }
 
@@ -1901,10 +1954,13 @@ bool KDirLister::doMimeFilter( const QString& mime, const QStringList& filters )
     return true;
 
   KMimeType::Ptr mimeptr = KMimeType::mimeType(mime);
+  //kdDebug(7004) << "doMimeFilter: investigating: "<<mimeptr->name()<<endl;
   QStringList::ConstIterator it = filters.begin();
   for ( ; it != filters.end(); ++it )
     if ( mimeptr->is(*it) )
       return true;
+    //else   kdDebug(7004) << "doMimeFilter: compared without result to  "<<*it<<endl;
+
 
   return false;
 }
@@ -1952,8 +2008,9 @@ void KDirLister::handleError( KIO::Job *job )
 void KDirLister::addNewItem( const KFileItem *item )
 {
   bool isNameFilterMatch = (d->dirOnlyMode && !item->isDir()) || !matchesFilter( item );
-  if (isNameFilterMatch)
-     return; // No reason to continue... bailing out here prevents a mimetype scan.
+
+  if ( isNameFilterMatch )
+    return; // No reason to continue... bailing out here prevents a mimetype scan.
 
   bool isMimeFilterMatch = !matchesMimeFilter( item );
 
@@ -1964,7 +2021,7 @@ void KDirLister::addNewItem( const KFileItem *item )
 
     d->lstNewItems->append( item );            // items not filtered
   }
-  else if ( !isNameFilterMatch )
+  else
   {
     if ( !d->lstMimeFilteredItems )
       d->lstMimeFilteredItems = new KFileItemList;
@@ -1980,6 +2037,17 @@ void KDirLister::addNewItems( const KFileItemList& items )
     addNewItem( *kit );
 }
 
+void KDirLister::aboutToRefreshItem( const KFileItem *item )
+{
+  bool isNameFilterMatch = (d->dirOnlyMode && !item->isDir()) || !matchesFilter( item );
+  bool isMimeFilterMatch = !matchesMimeFilter( item );
+
+  if ( !isNameFilterMatch && !isMimeFilterMatch )
+    d->refreshItemWasFiltered = false;
+  else
+    d->refreshItemWasFiltered = true;
+}
+
 void KDirLister::addRefreshItem( const KFileItem *item )
 {
   bool isNameFilterMatch = (d->dirOnlyMode && !item->isDir()) || !matchesFilter( item );
@@ -1987,15 +2055,29 @@ void KDirLister::addRefreshItem( const KFileItem *item )
 
   if ( !isNameFilterMatch && !isMimeFilterMatch )
   {
-    if ( !d->lstRefreshItems )
-      d->lstRefreshItems = new KFileItemList;
+    if ( d->refreshItemWasFiltered )
+    {
+      if ( !d->lstNewItems )
+        d->lstNewItems = new KFileItemList;
 
-    d->lstRefreshItems->append( item );
-  } else {
+      d->lstNewItems->append( item );
+    }
+    else
+    {
+      if ( !d->lstRefreshItems )
+        d->lstRefreshItems = new KFileItemList;
+
+      d->lstRefreshItems->append( item );
+    }
+  }
+  else if ( !d->refreshItemWasFiltered )
+  {
     if ( !d->lstRemoveItems )
       d->lstRemoveItems = new KFileItemList;
 
-      d->lstRemoveItems->append( item );//notify the user that the mimetype of a file changed, which doesn't match a filter or does match an exclude  filter
+    // notify the user that the mimetype of a file changed that doesn't match
+    // a filter or does match an exclude filter
+    d->lstRemoveItems->append( item );
   }
 }
 
@@ -2058,7 +2140,7 @@ void KDirLister::slotInfoMessage( KIO::Job *, const QString& message )
 
 void KDirLister::slotPercent( KIO::Job *job, unsigned long pcnt )
 {
-  d->jobData[static_cast<KIO::ListJob*>(job)].percent = pcnt;
+  d->jobData[static_cast<KIO::ListJob *>(job)].percent = pcnt;
 
   int result = 0;
 
@@ -2081,7 +2163,7 @@ void KDirLister::slotPercent( KIO::Job *job, unsigned long pcnt )
 
 void KDirLister::slotTotalSize( KIO::Job *job, KIO::filesize_t size )
 {
-  d->jobData[static_cast<KIO::ListJob*>(job)].totalSize = size;
+  d->jobData[static_cast<KIO::ListJob *>(job)].totalSize = size;
 
   KIO::filesize_t result = 0;
   QMap< KIO::ListJob *, KDirListerPrivate::JobData >::Iterator dataIt = d->jobData.begin();
@@ -2096,7 +2178,7 @@ void KDirLister::slotTotalSize( KIO::Job *job, KIO::filesize_t size )
 
 void KDirLister::slotProcessedSize( KIO::Job *job, KIO::filesize_t size )
 {
-  d->jobData[static_cast<KIO::ListJob*>(job)].processedSize = size;
+  d->jobData[static_cast<KIO::ListJob *>(job)].processedSize = size;
 
   KIO::filesize_t result = 0;
   QMap< KIO::ListJob *, KDirListerPrivate::JobData >::Iterator dataIt = d->jobData.begin();
@@ -2111,7 +2193,7 @@ void KDirLister::slotProcessedSize( KIO::Job *job, KIO::filesize_t size )
 
 void KDirLister::slotSpeed( KIO::Job *job, unsigned long spd )
 {
-  d->jobData[static_cast<KIO::ListJob*>(job)].speed = spd;
+  d->jobData[static_cast<KIO::ListJob *>(job)].speed = spd;
 
   int result = 0;
   QMap< KIO::ListJob *, KDirListerPrivate::JobData >::Iterator dataIt = d->jobData.begin();
@@ -2190,7 +2272,7 @@ KFileItemList KDirLister::itemsForDir( const KURL &dir, WhichItems which) const
 
 // to keep BC changes
 
-void KDirLister::virtual_hook( int, void* )
+void KDirLister::virtual_hook( int, void * )
 { /*BASE::virtual_hook( id, data );*/ }
 
 #include "kdirlister.moc"
