@@ -56,11 +56,14 @@ Value StringInstanceImp::get(ExecState *exec, const Identifier &propertyName) co
     return Number(internalValue().toString(exec).size());
 
   bool ok;
-  unsigned index = propertyName.toULong(&ok);
+  const unsigned index = propertyName.toArrayIndex(&ok);
   if (ok) {
-    UString str = internalValue().toString(exec);
-    if (index < (unsigned)str.size())
-      return String(str.substr(index,1));
+    const UString s = internalValue().toString(exec);
+    const unsigned length = s.size();
+    if (index < length) {
+      const UChar c = s[index];
+      return String(UString(&c, 1));
+    }
   }
 
   return ObjectImp::get(exec, propertyName);
@@ -151,7 +154,7 @@ const ClassInfo StringPrototypeImp::info = {"String", &StringInstanceImp::info,
 @end
 */
 // ECMA 15.5.4
-StringPrototypeImp::StringPrototypeImp(ExecState */*exec*/,
+StringPrototypeImp::StringPrototypeImp(ExecState * /*exec*/,
                                        ObjectPrototypeImp *objProto)
   : StringInstanceImp(objProto)
 {
@@ -187,7 +190,7 @@ Value StringProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &arg
 {
   Value result;
 
-  // toString and valueOf are no generic function.
+  // toString and valueOf are no generic functions.
   if (id == ToString || id == ValueOf) {
     KJS_CHECK_THIS( StringInstanceImp, thisObj );
 
@@ -539,19 +542,19 @@ Value StringProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &arg
     result = String("<SUP>" + s + "</SUP>");
     break;
   case Fontcolor:
-    result = String("<FONT COLOR=" + a0.toString(exec) + ">"
+    result = String("<FONT COLOR=\"" + a0.toString(exec) + "\">"
 		    + s + "</FONT>");
     break;
   case Fontsize:
-    result = String("<FONT SIZE=" + a0.toString(exec) + ">"
+    result = String("<FONT SIZE=\"" + a0.toString(exec) + "\">"
 		    + s + "</FONT>");
     break;
   case Anchor:
-    result = String("<a name=" + a0.toString(exec) + ">"
+    result = String("<a name=\"" + a0.toString(exec) + "\">"
 		    + s + "</a>");
     break;
   case Link:
-    result = String("<a href=" + a0.toString(exec) + ">"
+    result = String("<a href=\"" + a0.toString(exec) + "\">"
 		    + s + "</a>");
     break;
 #endif
@@ -611,7 +614,7 @@ Value StringObjectImp::call(ExecState *exec, Object &/*thisObj*/, const List &ar
 // ------------------------------ StringObjectFuncImp --------------------------
 
 // ECMA 15.5.3.2 fromCharCode()
-StringObjectFuncImp::StringObjectFuncImp(ExecState */*exec*/, FunctionPrototypeImp *funcProto)
+StringObjectFuncImp::StringObjectFuncImp(ExecState* /*exec*/, FunctionPrototypeImp *funcProto)
   : InternalFunctionImp(funcProto)
 {
   Value protect(this);
