@@ -1,5 +1,5 @@
 /*
- * $Id: highcolor.cpp,v 1.51 2003/06/25 22:32:16 mueller Exp $
+ * $Id: highcolor.cpp,v 1.53 2005/02/15 13:26:53 hasso Exp $
  *
  * KDE3 HighColor Style (version 1.0)
  * Copyright (C) 2001-2002 Karol Szwed      <gallium@kde.org>
@@ -76,7 +76,7 @@ class HighColorStylePlugin : public QStylePlugin
 		}
 };
 
-Q_EXPORT_PLUGIN( HighColorStylePlugin )
+KDE_Q_EXPORT_PLUGIN( HighColorStylePlugin )
 // ---------------------------------------------------
 
 
@@ -1842,7 +1842,7 @@ QSize HighColorStyle::sizeFromContents( ContentsType contents,
 
 			// Ensure we stick to standard width and heights.
 			if ( button->isDefault() || button->autoDefault() ) {
-				if ( w < 80 && !button->pixmap() )
+				if ( w < 80 && !button->text().isEmpty() )
 					w = 80;
 
 				if ( type != HighColor ) {
