@@ -31,7 +31,7 @@
    Boston, MA 02111-1307, USA.
 */
 
-// $Id: keramik.cpp,v 1.134.2.2 2004/11/28 17:00:50 giessl Exp $
+// $Id: keramik.cpp,v 1.138 2004/11/28 16:58:40 giessl Exp $
 
 #include <qbitmap.h>
 #include <qcheckbox.h>
@@ -92,7 +92,7 @@ public:
 	}
 };
 
-Q_EXPORT_PLUGIN( KeramikStylePlugin )
+KDE_Q_EXPORT_PLUGIN( KeramikStylePlugin )
 // ---------------------------------------------------
 
 
